package myCBR;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import parserXML.OverviewSpecifics;
import de.dfki.mycbr.core.DefaultCaseBase;
import de.dfki.mycbr.core.ICaseBase;
import de.dfki.mycbr.core.Project;
import de.dfki.mycbr.core.casebase.Instance;
import de.dfki.mycbr.core.model.Concept;
import de.dfki.mycbr.core.retrieval.Retrieval;
import de.dfki.mycbr.core.similarity.Similarity;
import de.dfki.mycbr.util.Pair;

public class AdaptArmy {
	
	private Project prj;
	private Concept unit;
	private Concept army;
	
	private DefaultCaseBase UnitsFromQueryCB;
	
	private OverviewSpecifics os;
	
	private ArmyInstance originalBestResult;
	private Similarity originalSimilarity;
	
	private List<UnitInstance> originalUnitInstances;
	
	private List<UnitInstance> removedUnitInstances;
	
	private List<Instance> removalUnitInstancesPostSimilarityAssessment; 
	
	private float desiredTotalPoints;
	private String desiredFaction;
	
	private List<UnitInstance> desiredUnits;
	private List<UnitInstance> unitsToKeep;
//	private List<UnitInstance> unitsNotAddedIntoSolution;
	
	
	private List<UnitInstance> cores;
	private List<UnitInstance> rares;
	private List<UnitInstance> specials;
	private List<UnitInstance> heroes;
	private List<UnitInstance> lords;
	
	
	
	private float corePointsSpent;
	private float rarePointsSpent;
	private float specialPointsSpent;
	private float heroPointsSpent;
	private float lordPointsSpent;
	
	private float totalPointsSpent;
	
	
	private float minimumCostOfUnit;
	private float minimumCostOfModel;
	
	private HashMap<String,UnitInstance> availableUnits;
	
	
	
	private HashMap<String,Integer> desiredUnitsCount;
	private HashMap<String,Integer> inArmyUnitCount;
	private HashMap<String,Integer> InArmyUnitCountTemp;
	
	private boolean enoughToAddAnotherUnit;
	private boolean enoughToIncreaseCountOfUnit;
	private boolean enoughtoAddUnitButLimited;
	private boolean canRemoveGearFromCore;
	private boolean possibleToReAddUnits;
	
	public AdaptArmy(Project prj, ArmyInstance queryResultArmy, Similarity querySim, String Faction, float totalPoints, List<UnitInstance> desiredUnits, HashMap<String, UnitInstance> availUnits, float minimumCost , OverviewSpecifics os){
		this.prj = prj;
		this.unit = prj.getConceptByID("Unit");
		this.army = prj.getConceptByID("Army");
		this.originalBestResult = queryResultArmy;
		this.originalSimilarity = querySim;
		this.desiredFaction = Faction;
		this.desiredTotalPoints = totalPoints;
		this.desiredUnits = desiredUnits;
		this.availableUnits = availUnits;
		this.setMinimumCostOfUnit(minimumCost);
		this.os = os;
		this.removedUnitInstances = new ArrayList<>();
		
		this.enoughToAddAnotherUnit = false;
		this.enoughToIncreaseCountOfUnit = false;
		this.canRemoveGearFromCore = true;
		this.enoughtoAddUnitButLimited = false;
	}
	
	public void runAdaptation(){
		System.out.println(originalSimilarity.getValue());
		
		populateQueryUnitInstances();
		populateDesiredUnitCount();
		populateInArmyUnitCount();
		findLowestModelCostOfUsedUnits();
		initPointValues();
		if((originalSimilarity.getValue() >= 0.999) && checkIfPointsSpentAreBelowTarget() && checkIfDesiredUnitsAreIncluded()){
			System.out.println("The query found identical case");
		}
		else{
			System.out.println("Adapting...");
//			populateQueryUnitInstances();
//			populateDesiredUnitCount();
//			populateInArmyUnitCount();
//			findLowestModelCostOfUsedUnits();
//			System.out.println();
//			initPointValues();

//			System.out.println(checkIfPointsAreWithinThresholds());
//			System.out.println(checkIfPointWithinThresholdPointLimit());
//			System.out.println(checkIfDesiredUnitsAreIncluded());
			
			if(!checkIfDesiredUnitsAreIncluded()){
				System.out.println("trying to add desired units into the thing");
				populateNewCBWithUnitsFromQuery();
				updateCaseWithDesiredUnits();
			}
			
			//TODO reAdd cores if needed
			addCoresBackIntoRoster();
			
			while(!coresWithinThresholdPointLimit()){
				addInAdditionalCores();
			}
			
			
			findLowestModelCostOfUsedUnits();
			
			printRosterStatus();
			System.out.println(getMinimumCostOfModel());
			
			if(checkIfDesiredUnitsAreIncluded() && checkIfPointsSpentAreCloseToTarget() && checkIfPointsAreWithinThresholds() && checkIfPointWithinThresholdPointLimit() && checkIfPointsSpentAreBelowTarget()){
				System.err.println("original (modified) case is good enough");
				printRosterStatus();
				//TODO print out case - store case somewhere.. idk
			}
			//TODO add missing desired units to the case / replace some existing unit
			else{
			//	Specified limit is slightly lower than the cases
				if(/*checkIfPointsAreWithinThresholds() && checkIfPointWithinThresholdPointLimit() &&*/ checkIfDesiredUnitsAreIncluded() /* && !checkIfPointsSpentAreBelowTarget()*/){
//					System.out.println("Adaption on a case with too much points spent, but with all desired units");
					System.out.println("attempting to lower a core count");
					int attempts = 25*cores.size();
					while(coresWithinThresholdPointLimit() /*&& !checkIfPointsSpentAreCloseToTarget()*/ && attempts>0){
						reduceCountOfRandomCore();
						attempts--;
					}
					attempts = 10*cores.size();
					System.out.println("attempting to remove a  core");
					while(coresWithinThresholdPointLimit() /*&& !checkIfPointsSpentAreCloseToTarget()*/ && attempts>0){
						removeRandomExtraCore();
						attempts--;
					}
					System.out.println("attempting to remove core gear");
					attempts = 10*cores.size();
					while(coresWithinThresholdPointLimit() /*&& !checkIfPointsSpentAreCloseToTarget()*/ && attempts>0 && isCanRemoveGearFromCore()){
						removeRandomGearFromCore();
						attempts--;
					}
					
					printRosterStatus();
					
					System.out.println("cannot reduce more values off cores");
					System.out.println(getTotalPointsSpent() + " " + getDesiredTotalPoints());
					System.out.println(checkIfPointsAreWithinThresholds() + " " + checkIfPointWithinThresholdPointLimit() + " " + checkIfDesiredUnitsAreIncluded() + " " + checkIfPointsSpentAreBelowTarget());
					
					if(checkIfPointsAreWithinThresholds() && checkIfPointWithinThresholdPointLimit() && checkIfDesiredUnitsAreIncluded() && checkIfPointsSpentAreBelowTarget() && checkIfPointsSpentAreCloseToTarget()){
						System.err.println("finished altering");
						printRosterStatus();
						return;
					}
					
					if(checkIfPointWithinThresholdPointLimit()){
						System.out.println("all pools within thresholds");
					}
					else{
						System.out.println("some pool not within threshold");
						System.out.println(lordsWithinThresholdPointLimit());
						if(!coresWithinThresholdPointLimit()){
							System.out.println(" core " + coresWithinThresholdPointLimit());
							System.err.println("This should never trigger");
							//TODO - should be handled above anyway
						}
						if(!raresWithinThresholdPointLimit()){
							System.out.println(" rare " + raresWithinThresholdPointLimit());
							attempts = 20;
							while(!raresWithinThresholdPointLimit() && attempts>0){
								reduceCountOfRandomRare();
								attempts--;
							}
							attempts = 20;
							while(!raresWithinThresholdPointLimit() && attempts>0){
								removeRandomGearFromRare();
								attempts--;
							}
							attempts = 20;
							while(!raresWithinThresholdPointLimit() && attempts>0){
								removeRandomRare();
								attempts--;
							}
							printRosterStatus();
						}
						if(!specialsWithinThresholdPointLimit()){
							System.out.println(" special " + specialsWithinThresholdPointLimit());
							attempts = 20;
							while(!specialsWithinThresholdPointLimit() && attempts>0){
								reduceCountOfRandomSpecial();
								attempts--;
							}
							attempts = 20;
							while(!specialsWithinThresholdPointLimit() && attempts>0){
								removeRandomGearFromSpecial();
								attempts--;
							}
							attempts = 20;
							while(!specialsWithinThresholdPointLimit() && attempts>0){
								removeRandomSpecial();
								attempts--;
							}
							printRosterStatus();
							//TODO
						}
						if(!heroesWithinThresholdPointLimit()){
							System.out.println(" hero " + heroesWithinThresholdPointLimit());
							attempts = 20;
							while(!heroesWithinThresholdPointLimit() && attempts>0){
								removeRandomGearFromHero();
								attempts--;
							}
							attempts = 20;
							while(!heroesWithinThresholdPointLimit() && attempts>0){
								removeRandomHero();
								attempts--;
							}
							printRosterStatus();
							//TODO
						}
						System.out.println("l " + lordsWithinThresholdPointLimit());
						if(!lordsWithinThresholdPointLimit()){
							System.out.println(" lord " + lordsWithinThresholdPointLimit());
							attempts = 20;
							while(!lordsWithinThresholdPointLimit() && attempts>0){
								removeRandomGearFromLord();
								attempts--;
							}
							attempts = 20;
							while(!lordsWithinThresholdPointLimit() && attempts>0){
								removeRandomLord();
								attempts--;
							}
							printRosterStatus();
							//TODO
						}
					}
					
					//TODO focus first on meeting the caps for rare --> special --> hero --> lord  ... (this applies mostly to rare with its 25% limit as opposed to 50% of others)
					
					System.out.println("Trying to reduce count of rare and special");
					attempts = 30;
					while(!checkIfPointsSpentAreBelowTarget() && attempts>0){
						reduceCountOfRandomRareOrSpecial();
						attempts--;
					}

					
					
					System.out.println("trying to remove Excess units that are close to the limit");
					attempts = 20;
					while(!checkIfPointsSpentAreBelowTarget() && attempts>0){
						boolean removed = removeRandomUnit();
						if(!removed)
							break;
						attempts--;
					}
					
					//TODO remove excess gear
					
					attempts = 200;
					while(!checkIfPointsSpentAreBelowTarget() && attempts>0){
						removeRandomGear();
						attempts--;
					}
					
					printRosterStatus();
				}
			}
			
			//TODO ensure its not possible to spend all points without adding a commander
			System.out.println("commander " + hasACommander());
			
			setPossibleToReAddUnits(true);
			
			int attempts = 100;
			
			if(!checkIfPointsSpentAreCloseToTarget() || !hasACommander()){
				System.out.println("need to add more shit - or a commander");
				while((!checkIfPointsSpentAreCloseToTarget() || !hasACommander()) && attempts>0){
					if(!hasACommander()){
						addCommanderToArmy();
						printRosterStatus();
					}
					else if(!removedUnitInstances.isEmpty() && isEnoughToAddAnotherUnit() && isPossibleToReAddUnits()){
						reAddARandomDeletedUnit();
						printRosterStatus();
					}
					else if (isEnoughToAddAnotherUnit()){
						System.out.println("now we just need to add more shit somewhere. idk where dude");
						System.out.println(isEnoughToAddAnotherUnit());
						addRandomUnit();
						printRosterStatus();
					}
					else if(isEnoughToIncreaseCountOfUnit()){
						System.out.println("at least we can increase count somewhere");
						increaseCountOfARandomUnit();
						printRosterStatus();
					}
					attempts--;
				}
			}
			
			System.out.println();
			System.out.println("points spent " + checkIfPointsSpentAreCloseToTarget());
			removeQueryUnitsCB();
		}
	}
	
	public void initPointValues(){
		corePointsSpent = 0;
		rarePointsSpent = 0;
		specialPointsSpent = 0;
		heroPointsSpent = 0;
		lordPointsSpent = 0;
		totalPointsSpent = 0;
		
		cores = new ArrayList<>();
		rares = new ArrayList<>();
		specials = new ArrayList<>();
		heroes = new ArrayList<>();
		lords = new ArrayList<>();
		
		for (UnitInstance unitInstance : originalUnitInstances) {
			addUnitToRoster(unitInstance);
		}
		printRosterStatus();
	}
	
	public void printRosterStatus(){
		System.out.println("core " + corePointsSpent + " \t" + cores);
		System.out.println("rare " + rarePointsSpent + " \t" + rares);
		System.out.println("special " + specialPointsSpent + " \t" + specials);
		System.out.println("hero " + heroPointsSpent + " \t" + heroes);
		System.out.println("lord " + lordPointsSpent + " \t" + lords );
		System.out.println("total " + totalPointsSpent + " \t" + inArmyUnitCount);
		System.out.println("query units \t" + desiredUnitsCount);
	}
	
	public void populateQueryUnitInstances(){
		originalUnitInstances = originalBestResult.getMultiUnitInstances();
		for (UnitInstance unitInstance : originalUnitInstances) {
			System.out.print(unitInstance.getUnitInstance() + " ");
			System.out.print(" extra pts " + unitInstance.getSurplussPoints());
			System.out.print(" modelCost " + unitInstance.getCostPerModel() + " " + unitInstance.getPointPerModelValue());
			System.out.print(" count " + unitInstance.getModelCountValue());
			System.out.print(" cost total " + unitInstance.getPointValue() + "\n");
		}
	}
	
	public void populateDesiredUnitCount(){
		desiredUnitsCount = new HashMap<>();
		for (UnitInstance unitInstance : desiredUnits) {
			if(desiredUnitsCount.containsKey(unitInstance.getName())){
				desiredUnitsCount.put(unitInstance.getName(), desiredUnitsCount.get(unitInstance.getName())+1);
			}
			else
				desiredUnitsCount.put(unitInstance.getName(), 1);
		}
		System.out.println(desiredUnitsCount);
	}
	
	public void populateInArmyUnitCount(){
		inArmyUnitCount = new HashMap<>();
		for (UnitInstance unitInstance : originalUnitInstances) {
			if(inArmyUnitCount.containsKey(unitInstance.getName())){
				inArmyUnitCount.put(unitInstance.getName(), inArmyUnitCount.get(unitInstance.getName())+1);
			}
			else
				inArmyUnitCount.put(unitInstance.getName(), 1);
		}
		System.out.println(inArmyUnitCount);
	}
	
	public boolean checkIfPointsAreWithinThresholds(){
		return coresWithinThreshold()&&raresWithinThreshold()&&specialsWithinThreshold()&&heroesWithinThreshold()&&lordsWithinThreshold();
	}
	
	public boolean checkIfPointWithinThresholdPointLimit(){
		return coresWithinThresholdPointLimit()&&raresWithinThresholdPointLimit()&&specialsWithinThresholdPointLimit()&&heroesWithinThresholdPointLimit()&&lordsWithinThresholdPointLimit();
	}
	
	public boolean checkIfDesiredUnitsAreIncluded(){
		boolean allFulFIlled = true;
//		System.out.println(inArmyUnitCount);
//		System.out.println(desiredUnitsCount);
		for (String s : desiredUnitsCount.keySet()) {
			if(inArmyUnitCount.containsKey(s)){
				if(inArmyUnitCount.get(s)<desiredUnitsCount.get(s)){
					allFulFIlled = false;
					System.err.println("too low count of " + s + " in army (" + inArmyUnitCount.get(s)+")<("+desiredUnitsCount.get(s)+")");
					break;
				}
//				else
//					System.err.println("enough " +s + " in army already");
			}
			else{
				allFulFIlled = false;
				System.err.println("Missing " + s + " in army");
				break;
			}
		}
		return allFulFIlled;
	}
	
	public boolean checkIfPointsSpentAreBelowTarget(){
		return totalPointsSpent<=desiredTotalPoints;
	}
		
	public boolean checkIfPointsSpentAreCloseToTarget(){
		boolean positive = false;
		float sparePoints = desiredTotalPoints-totalPointsSpent;
		if(sparePoints>=0){
			positive = true;
		}
		
		if(positive){
			if(sparePoints>minimumCostOfUnit){
//				System.out.println("can add another unit");
				if(!isEnoughtoAddUnitButLimited())
					setEnoughToAddAnotherUnit(true);
			}
			else{
				setEnoughToAddAnotherUnit(false);
				//TODO, check if you can increase the count of unit somewhere . ish complete
				if(sparePoints>getMinimumCostOfModel()){
//					System.out.println("Enough to increase the modelCount of at least one unit (" + sparePoints + ") > ("+ getMinimumCostOfModel()+ ")");
					setEnoughToIncreaseCountOfUnit(true);
					return false;
				}
				setEnoughToIncreaseCountOfUnit(false);
				return true;
			}
		}
		
		return false;
	}
	
	public void findLowestModelCostOfUsedUnits(){
		float lowest = 110000000;
		for (String s : inArmyUnitCount.keySet()) {
			if(availableUnits.get(s).getPointPerModelValue()<lowest){
				lowest = availableUnits.get(s).getPointPerModelValue();
			}
		}
		setMinimumCostOfModel(lowest);
	}
	
	/**
	 * Core
	 */
	
	public void reduceCountOfRandomCore(){
		int pos = (int) (Math.random()*cores.size());
		UnitInstance thisUnit = cores.get(pos);
		if(thisUnit.getModelCountValue()>(thisUnit.getMinSelections())){
			subtractCoreValues(thisUnit.getPointPerModelValue());
			System.out.println(thisUnit + " " + thisUnit.getModelCountValue() + " " + thisUnit.getPointPerModelValue() + " " +getCoresPercentage() + " " + getCoresPercentagePointLimit() + " " + thisUnit.getPointPerModelValue());
			if(coresWithinThresholdPointLimit()){
				System.out.println("reducing " + thisUnit.getUnitInstance());
				thisUnit.setModelCountValue(thisUnit.getModelCountValue()-1);
				thisUnit.setPointValue(thisUnit.getPointValue()-thisUnit.getPointPerModelValue());
			}
			else{
				addCoreValues(thisUnit.getPointPerModelValue());
			}
		}
	}
	
	public void removeRandomExtraCore(){
		int pos = (int) (Math.random()*cores.size());
		UnitInstance thisUnit = cores.get(pos);
//		System.out.println("trying to remove " + thisUnit + " " + thisUnit.getPointValue() + " " +getExtraCorePointsValue());
		if(thisUnit.getPointValue()<=(getExtraCorePointsValue()) && (inArmyUnitCount.get(thisUnit.getName())>getDesiredCount(thisUnit.getName()))){
			System.out.println("removing " + thisUnit.getUnitInstance());
			removeUnitFromRoster(thisUnit);
		}
	}
	
	public void removeRandomGearFromCore(){
		List<UnitInstance> canRemoveGear = new ArrayList<UnitInstance>();
		for (UnitInstance unitInstance : cores) {
			if(hasSurplussPoints(unitInstance)){
				canRemoveGear.add(unitInstance);
			}
		}
		if(!canRemoveGear.isEmpty()){
			
			UnitInstance removeInstance = canRemoveGear.get((int)(Math.random()*canRemoveGear.size()));
			
			float removing = 0;
			
			if(removeInstance.getSurplussPoints()>25){
				int dividableBy25 = (int) (removeInstance.getSurplussPoints()/25);
				
				float randomValueOf25Multiple = (float) ((int)(Math.random()*dividableBy25)*25);
				
				removing = randomValueOf25Multiple>0?randomValueOf25Multiple:25;
				
			}
			else{
				removing = removeInstance.getSurplussPoints();
			}
			
			if(removing > getExtraCorePointsValue()){
				System.err.println("Trying to remove gear that would break core values , fixing");
				System.out.println(removing);
				removing = getExtraCorePointsValue();
				System.out.println(removing);
				if(removing < 1)
					setCanRemoveGearFromCore(false);
			}
			
			removeGearFromUnitInRoster(removeInstance, removing);
			System.out.println("removing gear from core " +removeInstance + " pts " + removing + " " + getExtraCorePointsValue());
			System.out.println();
			System.out.println(getExtraPointsValue());
			System.out.println();
		}
		else{
			System.err.println("could not remove more gear from cores");
			setCanRemoveGearFromCore(false);
		}
	}
	
	
	/**
	 * Rare
	 */
	
	public void reduceCountOfRandomRare(){
		int pos = (int) (Math.random()*rares.size());
		UnitInstance thisUnit = rares.get(pos);
		System.out.println(thisUnit + " " + thisUnit.getMinSelections());
		if(thisUnit.getModelCountValue()>thisUnit.getMinSelections() && thisUnit.getMinSelections()>0){
			System.out.println("r can potentially change " + thisUnit);
			subtractRareValues(thisUnit.getPointPerModelValue());
			System.out.println(thisUnit.getUnitInstance() + " " + thisUnit.getModelCountValue() + " " + thisUnit.getPointPerModelValue() + " " +getCoresPercentage() + " " + getCoresPercentagePointLimit() + " " + thisUnit.getPointPerModelValue());
			if(coresWithinThresholdPointLimit()){
				System.out.println("reducing " + thisUnit.getUnitInstance());
				thisUnit.setModelCountValue(thisUnit.getModelCountValue()-1);
				thisUnit.setPointValue(thisUnit.getPointValue()-thisUnit.getPointPerModelValue());
			}
			else{
				addRareValues(thisUnit.getPointPerModelValue());
			}
		}
	}
	
	public void removeRandomGearFromRare(){
		List<UnitInstance> canRemoveGear = new ArrayList<UnitInstance>();
		for (UnitInstance unitInstance : rares) {
			if(hasSurplussPoints(unitInstance)){
				canRemoveGear.add(unitInstance);
			}
		}
		
		if(!canRemoveGear.isEmpty()){
			UnitInstance removeInstance = canRemoveGear.get((int)(Math.random()*canRemoveGear.size()));
			
			float removing = 0;
			
			if(removeInstance.getSurplussPoints()>25){
				int dividableBy25 = (int) (removeInstance.getSurplussPoints()/25);
				
				float randomValueOf25Multiple = (float) ((int)(Math.random()*dividableBy25)*25);
				
				removing = randomValueOf25Multiple>0?randomValueOf25Multiple:25;
				
			}
			else{
				removing = removeInstance.getSurplussPoints();
			}
			removeGearFromUnitInRoster(removeInstance, removing);
			
			System.out.println("removing gear from rare " +removeInstance + " pts " + removing);
			System.out.println();
			System.out.println(getExtraPointsValue());
			System.out.println();
		}
	}
	
	public void removeRandomRare(){
		List<UnitInstance> canRemove = new ArrayList<UnitInstance>();
		int topCount = 0;
		//Attempt to remove duplicates first
		for (UnitInstance unitInstance : rares) {
			if(inArmyUnitCount.get(unitInstance.getName())>topCount && inArmyUnitCount.get(unitInstance.getName())>getDesiredCount(unitInstance.getName())){
				topCount = inArmyUnitCount.get(unitInstance.getName());
			}
		}
		for (UnitInstance unitInstance : rares) {
			if(inArmyUnitCount.get(unitInstance.getName())==topCount && inArmyUnitCount.get(unitInstance.getName())>getDesiredCount(unitInstance.getName())){
				canRemove.add(unitInstance);
			}
		}
		
		UnitInstance removeInstance = canRemove.get((int)(Math.random()*canRemove.size()));
		removeUnitFromRoster(removeInstance);
		
		System.out.println("removing rare " +removeInstance);
		System.out.println();
		System.out.println(getExtraPointsValue());
		System.out.println();
	}
	
	/**
	 * Special
	 */
	
	public void reduceCountOfRandomSpecial(){
		int pos = (int) (Math.random()*specials.size());
		UnitInstance thisUnit = specials.get(pos);
		System.out.println(thisUnit + " " + thisUnit.getMinSelections());
		if(thisUnit.getModelCountValue()>thisUnit.getMinSelections() && thisUnit.getMinSelections()>0){
			System.out.println("s can potentially change " + thisUnit);
			subtractSpecialValues(thisUnit.getPointPerModelValue());
			System.out.println(thisUnit.getUnitInstance() + " " + thisUnit.getModelCountValue() + " " + thisUnit.getPointPerModelValue() + " " +getRaresPercentage() + " " + getRaresPercentagePointLimit() + " " + thisUnit.getPointPerModelValue());
			if(coresWithinThresholdPointLimit()){
				System.out.println("reducing " + thisUnit.getUnitInstance());
				thisUnit.setModelCountValue(thisUnit.getModelCountValue()-1);
				thisUnit.setPointValue(thisUnit.getPointValue()-thisUnit.getPointPerModelValue());
			}
			else{
				addSpecialValues(thisUnit.getPointPerModelValue());
			}
		}
	}
	
	public void removeRandomGearFromSpecial(){
		List<UnitInstance> canRemoveGear = new ArrayList<UnitInstance>();
		for (UnitInstance unitInstance : specials) {
			if(hasSurplussPoints(unitInstance)){
				canRemoveGear.add(unitInstance);
			}
		}
		
		if(!canRemoveGear.isEmpty()){
			UnitInstance removeInstance = canRemoveGear.get((int)(Math.random()*canRemoveGear.size()));
			
			float removing = 0;
			
			if(removeInstance.getSurplussPoints()>25){
				int dividableBy25 = (int) (removeInstance.getSurplussPoints()/25);
				
				float randomValueOf25Multiple = (float) ((int)(Math.random()*dividableBy25)*25);
				
				removing = randomValueOf25Multiple>0?randomValueOf25Multiple:25;
				
			}
			else{
				removing = removeInstance.getSurplussPoints();
			}
			removeGearFromUnitInRoster(removeInstance, removing);
			
			System.out.println("removing gear from special " +removeInstance + " pts " + removing);
			System.out.println();
			System.out.println(getExtraPointsValue());
			System.out.println();
		}
	}
	
	
	public void removeRandomSpecial(){
		List<UnitInstance> canRemove = new ArrayList<UnitInstance>();
		int topCount = 0;
		//Attempt to remove duplicates first
		for (UnitInstance unitInstance : specials) {
			if(inArmyUnitCount.get(unitInstance.getName())>topCount && inArmyUnitCount.get(unitInstance.getName())>getDesiredCount(unitInstance.getName())){
				topCount = inArmyUnitCount.get(unitInstance.getName());
			}
		}
		for (UnitInstance unitInstance : specials) {
			if(inArmyUnitCount.get(unitInstance.getName())==topCount && inArmyUnitCount.get(unitInstance.getName())>getDesiredCount(unitInstance.getName())){
				canRemove.add(unitInstance);
			}
		}
		
		UnitInstance removeInstance = canRemove.get((int)(Math.random()*canRemove.size()));
		removeUnitFromRoster(removeInstance);
		
		System.out.println("removing special" +removeInstance);
		System.out.println();
		System.out.println(getExtraPointsValue());
		System.out.println();
	}
	
	/**
	 * Hero
	 */
	
	public void removeRandomGearFromHero(){
		List<UnitInstance> canRemoveGear = new ArrayList<UnitInstance>();
		for (UnitInstance unitInstance : heroes) {
			if(hasSurplussPoints(unitInstance)){
				canRemoveGear.add(unitInstance);
			}
		}
		
		if(!canRemoveGear.isEmpty()){
			UnitInstance removeInstance = canRemoveGear.get((int)(Math.random()*canRemoveGear.size()));
			
			float removing = 0;
			
			if(removeInstance.getSurplussPoints()>25){
				int dividableBy25 = (int) (removeInstance.getSurplussPoints()/25);
				
				float randomValueOf25Multiple = (float) ((int)(Math.random()*dividableBy25)*25);
				
				removing = randomValueOf25Multiple>0?randomValueOf25Multiple:25;
				
			}
			else{
				removing = removeInstance.getSurplussPoints();
			}
			removeGearFromUnitInRoster(removeInstance, removing);
			
			System.out.println("removing gear from hero " +removeInstance + " pts " + removing);
			System.out.println();
			System.out.println(getExtraPointsValue());
			System.out.println();
		}
	}
	
	
	public void removeRandomHero(){
		List<UnitInstance> canRemove = new ArrayList<UnitInstance>();
		int topCount = 0;
		//Attempt to remove duplicates first
		for (UnitInstance unitInstance : heroes) {
			if(inArmyUnitCount.get(unitInstance.getName())>topCount && inArmyUnitCount.get(unitInstance.getName())>getDesiredCount(unitInstance.getName())){
				topCount = inArmyUnitCount.get(unitInstance.getName());
			}
		}
		for (UnitInstance unitInstance : heroes) {
			if(inArmyUnitCount.get(unitInstance.getName())==topCount && inArmyUnitCount.get(unitInstance.getName())>getDesiredCount(unitInstance.getName())){
				canRemove.add(unitInstance);
			}
		}
		
		UnitInstance removeInstance = canRemove.get((int)(Math.random()*canRemove.size()));
		removeUnitFromRoster(removeInstance);
		
		System.out.println("removing hero" +removeInstance);
		System.out.println();
		System.out.println(getExtraPointsValue());
		System.out.println();
	}
	
	/**
	 * lord
	 */
	
	public void removeRandomGearFromLord(){
		List<UnitInstance> canRemoveGear = new ArrayList<UnitInstance>();
		for (UnitInstance unitInstance : lords) {
			if(hasSurplussPoints(unitInstance)){
				canRemoveGear.add(unitInstance);
			}
		}
		
		if(!canRemoveGear.isEmpty()){
		
			UnitInstance removeInstance = canRemoveGear.get((int)(Math.random()*canRemoveGear.size()));
			
			float removing = 0;
			
			if(removeInstance.getSurplussPoints()>25){
				int dividableBy25 = (int) (removeInstance.getSurplussPoints()/25);
				
				float randomValueOf25Multiple = (float) ((int)(Math.random()*dividableBy25)*25);
				
				removing = randomValueOf25Multiple>0?randomValueOf25Multiple:25;
				
			}
			else{
				removing = removeInstance.getSurplussPoints();
			}
			removeGearFromUnitInRoster(removeInstance, removing);
			
			System.out.println("removing gear from lord " +removeInstance + " pts " + removing);
			System.out.println();
			System.out.println(getExtraPointsValue());
			System.out.println();
		}
	}
		
	public void removeRandomLord(){
		List<UnitInstance> canRemove = new ArrayList<UnitInstance>();
		int topCount = 0;
		//Attempt to remove duplicates first
		for (UnitInstance unitInstance : lords) {
			if(inArmyUnitCount.get(unitInstance.getName())>topCount && inArmyUnitCount.get(unitInstance.getName())>getDesiredCount(unitInstance.getName())){
				topCount = inArmyUnitCount.get(unitInstance.getName());
			}
		}
		for (UnitInstance unitInstance : lords) {
			if(inArmyUnitCount.get(unitInstance.getName())==topCount && inArmyUnitCount.get(unitInstance.getName())>getDesiredCount(unitInstance.getName())){
				canRemove.add(unitInstance);
			}
		}
		
		UnitInstance removeInstance = canRemove.get((int)(Math.random()*canRemove.size()));
		removeUnitFromRoster(removeInstance);
		
		System.out.println("removing hero" +removeInstance);
		System.out.println();
		System.out.println(getExtraPointsValue());
		System.out.println();
	}
	
	
	
	/**
	 * Reduce count of a random rare or special
	 */
	
	
	public void reduceCountOfRandomRareOrSpecial(){
		boolean rare = (Math.random()>=0.5)?true:false;
		if(rare && rares.size()==0){
			rare = !rare;
		}
		else if(!rare && specials.size()==0){
			rare = !rare;
		}
				
		if(rare){
			reduceCountOfRandomRare();
		}
		else {
			reduceCountOfRandomSpecial();
		}
	}
	
	
	/**
	 * Remove a random unit from the current list
	 */
	
	public boolean removeRandomUnit(){
		boolean ret = false;
		List<UnitInstance> canRemove = new ArrayList<UnitInstance>();
		for (UnitInstance unitInstance : originalUnitInstances) {
			if(inArmyUnitCount.get(unitInstance.getName())>getDesiredCount(unitInstance.getName()) && !cores.contains(unitInstance)){
				canRemove.add(unitInstance);
			}
		}
		float surplussValue = getExtraPointsValue();
		float lowest = 5000000;
		UnitInstance lowestInstance = null;
		float bestPercentage = 100;
		float closestPercentage = 100000;
		UnitInstance closestInstance = null;
		//Try to find the closest value above the extra points. or go with the lowest
		for (UnitInstance unitInstance : canRemove) {
			float thisUnitValue = unitInstance.getPointValue();
			float thisUnitPercentage = (thisUnitValue/surplussValue)*100;
			
			System.out.println(unitInstance + " " + thisUnitValue + " " + thisUnitPercentage + " " + surplussValue);
			if(thisUnitValue<lowest){
				lowest = thisUnitValue;
				lowestInstance = unitInstance;
			}
			if(thisUnitPercentage<closestPercentage && thisUnitPercentage>bestPercentage){
				closestPercentage = thisUnitPercentage;
				closestInstance = unitInstance;
			}
		}
		
		float closestPercentDiffValue = ((closestPercentage-bestPercentage)/bestPercentage)*surplussValue;
		UnitInstance removeInstance = null;
		if(closestPercentDiffValue<lowest){
			removeInstance = closestInstance;
		}
		else
			removeInstance = lowestInstance;
		System.out.println("% " + closestPercentage + " val " + closestPercentDiffValue);
		System.out.println("removing a random unit " +removeInstance);
		if(removeInstance != null){
			ret= true;
			removeUnitFromRoster(removeInstance);
			printRosterStatus();
		}
		else{
			System.err.println("Could not remove another unit");
		}
		System.out.println();
		return ret;
	}
	
	/**
	 * Remove a random piece of gear
	 */
	
	public void removeRandomGear(){
		switch ((int)(Math.random()*5)) {
		case 0:
			System.out.println("core");
			removeRandomGearFromCore();
			break;
		case 1:
			System.out.println("rare");
			removeRandomGearFromRare();
			break;
		case 2:
			System.out.println("special");
			removeRandomGearFromSpecial();
			break;
		case 3:
			System.out.println("hero");
			removeRandomGearFromHero();
			break;
		case 4:
			System.out.println("lord");
			removeRandomGearFromLord();
			break;
		default:
			break;
		}
	}
	
	
	/**
	 * remove a unit from roster
	 * @param unitInstance
	 */
	
	public void removeUnitFromRoster(UnitInstance unitInstance){
		if(unitInstance.getArmyType().equals(os.getCore())){
			subtractCoreValues(unitInstance.getPointValue());
			cores.remove(unitInstance);
		}
		else if(unitInstance.getArmyType().equals(os.getRare())){
			subtractRareValues(unitInstance.getPointValue());
			rares.remove(unitInstance);
		}
		else if(unitInstance.getArmyType().equals(os.getSpecial())){
			subtractSpecialValues(unitInstance.getPointValue());
			specials.remove(unitInstance);
		}
		else if(unitInstance.getArmyType().equals(os.getHero())){
			subtractHeroValues(unitInstance.getPointValue());
			heroes.remove(unitInstance);
		}
		else if(unitInstance.getArmyType().equals(os.getLord())){
			subtractLordValues(unitInstance.getPointValue());
			lords.remove(unitInstance);
		}
		originalUnitInstances.remove(unitInstance);
		removedUnitInstances.add(unitInstance);
		inArmyUnitCount.put(unitInstance.getName(), inArmyUnitCount.get(unitInstance.getName()) -1);
		cleanInArmyCount();
	}
	
	/**
	 * Add unit to roster
	 * @param unitInstance
	 */
	
	public void addUnitToRoster(UnitInstance unitInstance){
		if(unitInstance.getArmyType().equals(os.getCore())){
			addCoreValues(unitInstance.getPointValue());
			cores.add(unitInstance);
		}
		else if(unitInstance.getArmyType().equals(os.getRare())){
			addRareValues(unitInstance.getPointValue());
			rares.add(unitInstance);
		}
		else if(unitInstance.getArmyType().equals(os.getSpecial())){
			addSpecialValues(unitInstance.getPointValue());
			specials.add(unitInstance);
		}
		else if(unitInstance.getArmyType().equals(os.getHero())){
			addHeroValues(unitInstance.getPointValue());
			heroes.add(unitInstance);
		}
		else if(unitInstance.getArmyType().equals(os.getLord())){
			addLordValues(unitInstance.getPointValue());
			lords.add(unitInstance);
		}
	}
	
	public void updateInArmyUnits(UnitInstance unitInstance){
		if(inArmyUnitCount.containsKey(unitInstance.getName())){
			inArmyUnitCount.put(unitInstance.getName(), inArmyUnitCount.get(unitInstance.getName()) + 1);
		}
		else
			inArmyUnitCount.put(unitInstance.getName(), 1);
	}
	
	/**
	 * ReAdd cores to get over threshold
	 */
	
	public void addCoresBackIntoRoster(){
		List<UnitInstance> coresAvailable = new ArrayList<>();
		for (UnitInstance unitInstance : removedUnitInstances) {
			if(unitInstance.getArmyType().equals(os.getCore()))
				coresAvailable.add(unitInstance);
		}
		
		System.out.println(coresAvailable);
		
		//TODO - currently just adds core back if it is worth more than the remaining to get it back over the cap. Or the first in the list. don't what do
		while(!coresWithinThresholdPointLimit() && !coresAvailable.isEmpty()){
			UnitInstance addBack = coresAvailable.get(0);
			System.out.println(getExtraCorePointsValue());
			float initDiff = addBack.getPointValue() + getExtraCorePointsValue();
			for (UnitInstance unitInstance : coresAvailable) {
				float thisDiff = unitInstance.getPointValue() + getExtraCorePointsValue();
				System.out.println(initDiff + " " + thisDiff);
				if((thisDiff < initDiff && thisDiff>-1) || (thisDiff > initDiff && initDiff<0)){
					initDiff = thisDiff;
					addBack = unitInstance;
				}
			}
			System.out.println(addBack + " " + initDiff);
			addUnitToRoster(addBack);
			updateInArmyUnits(addBack);
			removedUnitInstances.remove(addBack);
			coresAvailable.remove(addBack);
		}
	}
	
	
	/**
	 * Add more of the same kind of cores already in the roster. Chooses unit there are the least of first
	 */
	
	public void addInAdditionalCores(){
		
		List<UnitInstance> lowestCoreCountAlreadyInCase = new ArrayList<>();
		int count = 10000;
		for (String s : availableUnits.keySet()) {
			if(availableUnits.get(s).getArmyType().equals(os.getCore())){
				if(inArmyUnitCount.containsKey(s)){
					if(inArmyUnitCount.get(s) < count && availableUnits.get(s).getMaxInRoster()<inArmyUnitCount.get(s)){
						count = inArmyUnitCount.get(s);
						lowestCoreCountAlreadyInCase.clear();
						lowestCoreCountAlreadyInCase.add(availableUnits.get(s));
					}
					else if(inArmyUnitCount.get(s) == count)
						lowestCoreCountAlreadyInCase.add(availableUnits.get(s));
				}
			}
		}
		int pos = (int)(Math.random()*lowestCoreCountAlreadyInCase.size());
		System.out.println(pos + " " +lowestCoreCountAlreadyInCase);
		addUnitToRoster(lowestCoreCountAlreadyInCase.get(pos));
		updateInArmyUnits(lowestCoreCountAlreadyInCase.get(pos));
	}
	
	
	/**
	 * ReAdd the deleted unit with cost closest to missing
	 */
	
	public void reAddARandomDeletedUnit(){
		UnitInstance addBack = null;
		float best = 0;
		float target = Math.abs(getExtraPointsValue());
		for (UnitInstance unitInstance : removedUnitInstances) {
			if(unitInstance.getPointValue()>best && unitInstance.getPointValue()<=target && !checkIfAddingUnitWillBreakLimits(unitInstance)){ //TODO check if adding back will fuck up limits
				best = unitInstance.getPointValue();
				addBack = unitInstance;
				System.out.println("set best readdUnit");
			}
		}
		if(addBack==null){
			setPossibleToReAddUnits(false);
			System.err.println("impossible to readd a unit " + removedUnitInstances);
		}
		else{
			System.out.println(addBack + " " + removedUnitInstances);
			addUnitToRoster(addBack);
			updateInArmyUnits(addBack);
			removedUnitInstances.remove(addBack);
			System.out.println("readded " + addBack + " " + best);
		}
	}
	
	/**
	 * add a commander closest to the point limit, electing to add one of the same type that was in the original case if possible
	 */
	
	public void addCommanderToArmy(){
		System.out.println("Trying to add commander from previous case");
		List<UnitInstance> heroesAndLords = new ArrayList<>();
		for (UnitInstance unitInstance : removedUnitInstances) {
			if(unitInstance.getArmyType().equals(os.getLord()) || unitInstance.getArmyType().equals(os.getHero())){
				heroesAndLords.add(unitInstance);
			}
		}
		float target = getExtraPointsValue();
		System.out.println(heroesAndLords);
		System.out.println(target);
		boolean thereIsPointsForCommander = false;
		float best = 0;
		if(target<0){
			thereIsPointsForCommander = true;
			target = Math.abs(target);
		}
		UnitInstance getBackInThere = null; 
		if(thereIsPointsForCommander){
			for (UnitInstance unitInstance : heroesAndLords) {
				if(unitInstance.getPointValue()>best && unitInstance.getPointValue()<=target){
					best = unitInstance.getPointValue();
					getBackInThere = unitInstance;
				}
			}
		}
		if(getBackInThere!=null){
			addUnitToRoster(getBackInThere);
			updateInArmyUnits(getBackInThere);
			removedUnitInstances.remove(getBackInThere);
		}
		else{
			System.out.println("need to add a basic commander instead");
			best = 0;
			for (UnitInstance unitInstance : heroesAndLords) {
				if(availableUnits.get(unitInstance.getName()).getPointValue()<target && availableUnits.get(unitInstance.getName()).getPointValue()>best){
					best = availableUnits.get(unitInstance.getName()).getPointValue();
					getBackInThere = availableUnits.get(unitInstance.getName());
				}
			}
			if(getBackInThere==null){
				System.out.println("still not anything that works, selecting from all commanders");
				best = 0;
				for (String s : availableUnits.keySet()) {
					if(availableUnits.get(s).getArmyType().equals(os.getLord()) || availableUnits.get(s).getArmyType().equals(os.getHero())){
						if(availableUnits.get(s).getPointValue()<target && availableUnits.get(s).getPointValue()>best){
							best = availableUnits.get(s).getPointValue();
							getBackInThere = availableUnits.get(s);
						}
					}
				}
			}
			addUnitToRoster(getBackInThere);
			updateInArmyUnits(getBackInThere);
		}
		System.out.println(" added commander " + getBackInThere + " " + best);
	}
	
	/**
	 * Add unit, chooses a unit already existing in the roster if possible. Random unit amongst those there are fewest of.
	 */
	
	public void addRandomUnit(){
		List<UnitInstance> lowestCountAlreadyInCase = new ArrayList<>();
		int count = 10000;
		float target = Math.abs(getExtraPointsValue());
		for (String s : availableUnits.keySet()) {
			if(!availableUnits.get(s).getArmyType().equals(os.getCore())){
				if(inArmyUnitCount.containsKey(s)){
					if(inArmyUnitCount.get(s) < count && (availableUnits.get(s).getPointValue()<=target) && !checkIfAddingUnitWillBreakLimits(availableUnits.get(s))){
						count = inArmyUnitCount.get(s);
						lowestCountAlreadyInCase.clear();
						System.out.println((availableUnits.get(s).getPointValue()<=target) + " " + target + " " + availableUnits.get(s));
						lowestCountAlreadyInCase.add(availableUnits.get(s));
					}
					else if(inArmyUnitCount.get(s) == count && availableUnits.get(s).getPointValue()<=target && !checkIfAddingUnitWillBreakLimits(availableUnits.get(s)))
						lowestCountAlreadyInCase.add(availableUnits.get(s));
				}
			}
		}
		if(lowestCountAlreadyInCase.isEmpty()){
			System.err.println("cant add unit already in the thingy");
			setEnoughToAddAnotherUnit(false);
			setEnoughtoAddUnitButLimited(true);
			setEnoughToIncreaseCountOfUnit(true);
		}
		else{
			int pos = (int)(Math.random()*lowestCountAlreadyInCase.size());
			System.out.println(pos + " " +lowestCountAlreadyInCase);
			addUnitToRoster(lowestCountAlreadyInCase.get(pos));
			updateInArmyUnits(lowestCountAlreadyInCase.get(pos));
		}
	}
	
	/**
	 * Increase the Count of a random Unit, preferring rare and special over core.
	 */
	
	public void increaseCountOfARandomUnit(){
		List<UnitInstance> increaseAbleUnits = new ArrayList<>();
		for (UnitInstance unitInstance : originalUnitInstances) {
			if(unitInstance.getMinSelections()>0 && !unitInstance.getArmyType().equals(os.getCore()) && !checkIfIncreasingCountWillBreakLimits(unitInstance)){
				System.out.println(unitInstance + " " +unitInstance.getMinSelections());
				increaseAbleUnits.add(unitInstance);
			}
		}
		boolean tooExpensive = true;
		float target = Math.abs(getExtraPointsValue());
		for (UnitInstance unitInstance : increaseAbleUnits) {
			if(unitInstance.getPointPerModelValue()<target)
				tooExpensive = false;
		}
		
		if(increaseAbleUnits.isEmpty() || tooExpensive){
			for (UnitInstance unitInstance : cores) {
				if(unitInstance.getMinSelections()>0)
					increaseAbleUnits.add(unitInstance);
			}
		}
		float best = 0;
		UnitInstance countBack = null;
		for (UnitInstance unitInstance : increaseAbleUnits) {
			if(unitInstance.getPointPerModelValue()>best && unitInstance.getPointPerModelValue()<=target && !checkIfIncreasingCountWillBreakLimits(unitInstance)){
				best = unitInstance.getPointPerModelValue();
				countBack = unitInstance;
			}
		}
		
		System.out.println("increasing count of " + countBack);
		increaseCountForUnit(countBack);
	}
	
	public void increaseCountForUnit(UnitInstance unitInstance){
		if(unitInstance.getArmyType().equals(os.getCore())){
			addCoreValues(unitInstance.getPointPerModelValue());
		}
		else if(unitInstance.getArmyType().equals(os.getRare())){
			addRareValues(unitInstance.getPointPerModelValue());
		}
		else if(unitInstance.getArmyType().equals(os.getSpecial())){
			addSpecialValues(unitInstance.getPointPerModelValue());
		}
		else if(unitInstance.getArmyType().equals(os.getHero())){
			addHeroValues(unitInstance.getPointPerModelValue());
		}
		else if(unitInstance.getArmyType().equals(os.getLord())){
			addLordValues(unitInstance.getPointPerModelValue());
		}
		unitInstance.setModelCountValue(unitInstance.getModelCountValue()+1);
		unitInstance.setPointValue(unitInstance.getPointValue()+unitInstance.getPointPerModelValue());
	}
	

	
	/**
	 * Remove x amount of gear from a unit
	 * @param unitInstance, amount
	 */
	
	public void removeGearFromUnitInRoster(UnitInstance unitInstance, float amount){
		if(unitInstance.getArmyType().equals(os.getCore())){
			subtractCoreValues(amount);
		}
		else if(unitInstance.getArmyType().equals(os.getRare())){
			subtractRareValues(amount);
		}
		else if(unitInstance.getArmyType().equals(os.getSpecial())){
			subtractSpecialValues(amount);
		}
		else if(unitInstance.getArmyType().equals(os.getHero())){
			subtractHeroValues(amount);
		}
		else if(unitInstance.getArmyType().equals(os.getLord())){
			subtractLordValues(amount);
		}
		unitInstance.setSurplussPoints(unitInstance.getSurplussPoints()-amount);
		unitInstance.setPointValue(unitInstance.getPointValue()-amount);
	}
	
	public void populateNewCBWithUnitsFromQuery(){
		try {
			prj.createDefaultCB("UnitsInArmy");
			UnitsFromQueryCB = (DefaultCaseBase) prj.getCB("UnitsInArmy");
			
			for (UnitInstance unitInstance : originalUnitInstances) {			
				UnitsFromQueryCB.addCase(unitInstance.getUnitInstance());
			}
			
			System.out.println(prj.getCB("UnitsInArmy").getCases());
		} catch (Exception e) {
			System.err.println("Could not create the CB");
			e.printStackTrace();
		}
	}
	
	public void removeQueryUnitsCB(){
		prj.deleteCaseBase("UnitsInArmy");
	}
	
	/**
	 * add in all the desired units
	 */
	
	public void updateCaseWithDesiredUnits(){
		//TODO init list of units to remove. add units that should be added. Finally remove the dudes in the remove list.
		removalUnitInstancesPostSimilarityAssessment = new ArrayList<>();
		InArmyUnitCountTemp = new HashMap<>();
		
		for (String s : inArmyUnitCount.keySet()) {
			InArmyUnitCountTemp.put(s, inArmyUnitCount.get(s));
		}
		
		for (String s : desiredUnitsCount.keySet()) {
			while(!InArmyUnitCountTemp.containsKey(s)){
				perFormRetrievalOfMostSimilarUnit(s);
				inArmyUnitCount.put(s, 1);
				InArmyUnitCountTemp.put(s,  1);
			}
			while(InArmyUnitCountTemp.get(s)<desiredUnitsCount.get(s)){
				System.out.println("pre " + s + " "  + InArmyUnitCountTemp );
				perFormRetrievalOfMostSimilarUnit(s);
				inArmyUnitCount.put(s, inArmyUnitCount.get(s)+1);
				InArmyUnitCountTemp.put(s, InArmyUnitCountTemp.get(s)+1);
			}
		}
		printRosterStatus();
		List<UnitInstance> remList = new ArrayList<>();
		for (UnitInstance unitInstance : originalUnitInstances) {
			if(removalUnitInstancesPostSimilarityAssessment.contains(unitInstance.getUnitInstance())){
				remList.add(unitInstance);
			}
		}
		for (UnitInstance unitInstance : remList) {
			removeUnitFromRoster(unitInstance);
		}
		printRosterStatus();
	}
	
	/**
	 * find most similar units
	 * @param nameOfUnit
	 */
	
	public void perFormRetrievalOfMostSimilarUnit(String nameOfUnit){
		Retrieval r = new Retrieval(unit, UnitsFromQueryCB);
		Instance rInstance = r.getQueryInstance();
		
		UnitInstance retInstance = new UnitInstance(prj);
		
		retInstance.usePreviousUnit(unit, rInstance);
		retInstance.initEmptyUnit();
		
		UnitInstance modelInstance = availableUnits.get(nameOfUnit);
		
		retInstance.InitFromOtherInstance(modelInstance.getUnitInstance());
		
//		System.out.println(retInstance.getUnitInstance().getAttributes());
//		System.out.println(modelInstance.getUnitInstance().getAttributes());
		
		r.run();
		Pair<Instance, Similarity> best = getBestResult(r);
		if(best==null)
			System.err.println("Could not replace anything");
		else{
			removalUnitInstancesPostSimilarityAssessment.add(best.getFirst());
		}
		System.out.println(modelInstance.getName() + " " + best);
		addUnitToRoster(modelInstance);
		originalUnitInstances.add(modelInstance);
		System.out.println(" to be removed " + removalUnitInstancesPostSimilarityAssessment);
	}
	
	public void printResult(Retrieval r){
		for (Map.Entry<Instance, Similarity> entry: r.entrySet()) {
			if(entry.getValue().getValue()>0.0){
				System.out.println("Similarity: " + entry.getValue().getValue()
						+ " to case: " + entry.getKey().getName());
				System.out.println(entry.getKey().getAttributes());
				System.out.println();
			}
		}
	}
	
	public Pair<Instance,Similarity> getBestResult(Retrieval r){
		Instance best = null;
		Similarity bestSim = null;
		UnitInstance bestInstanceTemp = null;
		for (Map.Entry<Instance, Similarity> entry: r.entrySet()) {
			UnitInstance e = new UnitInstance(prj);
			e.usePreviousUnit(unit, entry.getKey());
//			System.err.print(e.getName() + " " +best);
			boolean isInArmy = InArmyUnitCountTemp.containsKey(e.getName());
			boolean isInDesired = desiredUnitsCount.containsKey(e.getName());
//			System.err.print(" " + isInArmy  + " " + InArmyUnitCountTemp.get(e.getName()) + " " +isInDesired + " " + desiredUnitsCount.get(e.getName()) + " ");
			if(entry.getValue().getValue()>0.0){
				if(best==null && !removalUnitInstancesPostSimilarityAssessment.contains(entry.getKey())){
					boolean canAdd = true;
					if(isInArmy && isInDesired)
						if(InArmyUnitCountTemp.get(e.getName())<=desiredUnitsCount.get(e.getName()))
							canAdd = false;
					if(canAdd){
						best = entry.getKey();
						bestSim = entry.getValue();
						bestInstanceTemp = e;
//						System.err.print(" set");
					}
				}
				else if (best!=null && !removalUnitInstancesPostSimilarityAssessment.contains(entry.getKey())){
					if(isInArmy&& isInDesired){
						if(bestSim.getValue()<entry.getValue().getValue() && (InArmyUnitCountTemp.get(e.getName())>desiredUnitsCount.get(e.getName()))){
							best = entry.getKey();
							bestSim = entry.getValue();
							bestInstanceTemp = e;
						}						
					}
					else{
						if(bestSim.getValue()<entry.getValue().getValue()){
							best = entry.getKey();
							bestSim = entry.getValue();
							bestInstanceTemp = e;
						}	
					}
				}
			}
//			System.err.print("\n");
		}
		if(best == null)
			return null;
		InArmyUnitCountTemp.put(bestInstanceTemp.getName(), InArmyUnitCountTemp.get(bestInstanceTemp.getName())-1);
		System.out.println(InArmyUnitCountTemp);
		Pair<Instance,Similarity> retPair = new Pair<Instance, Similarity>(best,bestSim);
		return retPair;
	}
	
	public int getDesiredCount(String instance){
		if(desiredUnitsCount.containsKey(instance)){
			return desiredUnitsCount.get(instance);
		}
		else
			return 0;
		
	}
	
	public void cleanInArmyCount(){
		List<String> removals = new ArrayList<String>();
		for (String s : inArmyUnitCount.keySet()) {
			if(inArmyUnitCount.get(s) == 0){
				removals.add(s);
			}
		}
		for (String string : removals) {
			inArmyUnitCount.remove(string);
		}
	}
	
	public float getExtraPointsValue(){
		return totalPointsSpent-desiredTotalPoints;
	}
	
	public float getExtraCorePointsValue(){
		float res = ((getCoresPercentagePointLimit()-os.getMinCorePercentage())/100)*getDesiredTotalPoints();
		return Math.round(res);
//		System.out.println(res +" " + Math.round(res) + " " + Math.round(-0.00000000234) + " " + Math.round(-123.99999999));
//		if(res<-1)
//			return (float) Math.floor(res);
//		return (float) Math.ceil(res);
	}
	
	public boolean hasSurplussPoints(UnitInstance instance){
		return instance.getSurplussPoints()>0;
	}
	
	public boolean hasACommander(){
		return !heroes.isEmpty() || !lords.isEmpty();
	}
	
	
	public void subtractCoreValues(float sub){
		corePointsSpent-=sub;
		totalPointsSpent-=sub;
	}
	
	public void addCoreValues(float add){
		corePointsSpent+=add;
		totalPointsSpent+=add;
	}
	
	public void subtractRareValues(float sub){
		rarePointsSpent-=sub;
		totalPointsSpent-=sub;
	}
	
	public void addRareValues(float add){
		rarePointsSpent+=add;
		totalPointsSpent+=add;
	}
	
	public void subtractSpecialValues(float sub){
		specialPointsSpent-=sub;
		totalPointsSpent-=sub;
	}
	
	public void addSpecialValues(float add){
		specialPointsSpent+=add;
		totalPointsSpent+=add;
	}
	
	public void subtractHeroValues(float sub){
		heroPointsSpent-=sub;
		totalPointsSpent-=sub;
	}
	
	public void addHeroValues(float add){
		heroPointsSpent+=add;
		totalPointsSpent+=add;
	}
	
	public void subtractLordValues(float sub){
		lordPointsSpent-=sub;
		totalPointsSpent-=sub;
	}
	
	public void addLordValues(float add){
		lordPointsSpent+=add;
		totalPointsSpent+=add;
	}
	
	
	
	public float getCoresPercentage(){
		return (corePointsSpent/totalPointsSpent)*100;
	}
	
	public float getRaresPercentage(){
		return (rarePointsSpent/totalPointsSpent)*100;
	}
	
	public float getSpecialsPercentage(){
		return (specialPointsSpent/totalPointsSpent)*100;
	}
	
	public float getHeroesPercentage(){
		return (heroPointsSpent/totalPointsSpent)*100;
	}
	
	public float getLordsPercentage(){
		return (lordPointsSpent/totalPointsSpent)*100;
	}
	
	
	
	
	public float getCoresPercentagePointLimit(){
		return (corePointsSpent/desiredTotalPoints)*100;
	}
	
	public float getRaresPercentagePointLimit(){
		return (rarePointsSpent/desiredTotalPoints)*100;
	}
	
	public float getSpecialsPercentagePointLimit(){
		return (specialPointsSpent/desiredTotalPoints)*100;
	}
	
	public float getHeroesPercentagePointLimit(){
		return (heroPointsSpent/desiredTotalPoints)*100;
	}
	
	public float getLordsPercentagePointLimit(){
		return (lordPointsSpent/desiredTotalPoints)*100;
	}
	
	
	
	
	public boolean coresWithinThreshold(){
		return getCoresPercentage()>=os.getMinCorePercentage();
	}
	
	public boolean raresWithinThreshold(){
		return getRaresPercentage()<=os.getMaxRarePercentage();
	}
	
	public boolean specialsWithinThreshold(){
		return getSpecialsPercentage()<=os.getMaxSpecialPercentage();
	}
	
	public boolean heroesWithinThreshold(){
		return getHeroesPercentage()<=os.getMaxHeroesPercentage();
	}
	
	public boolean lordsWithinThreshold(){
		return getLordsPercentage()<=os.getMaxLordsPercentage();
	}
	
	
	
	public boolean coresWithinThresholdPointLimit(){
		return getCoresPercentagePointLimit()>=os.getMinCorePercentage();
	}
	
	public boolean raresWithinThresholdPointLimit(){
		return getRaresPercentagePointLimit()<=os.getMaxRarePercentage();
	}
	
	public boolean specialsWithinThresholdPointLimit(){
		return getSpecialsPercentagePointLimit()<=os.getMaxSpecialPercentage();
	}
	
	public boolean heroesWithinThresholdPointLimit(){
		return getHeroesPercentagePointLimit()<=os.getMaxHeroesPercentage();
	}
	
	public boolean lordsWithinThresholdPointLimit(){
		return getLordsPercentagePointLimit()<=os.getMaxLordsPercentage();
	}
	
	
	public boolean checkIfIncreasingCountWillBreakLimits(UnitInstance unitInstance){
		if(unitInstance.getArmyType().equals(os.getCore())){
			float val = ((unitInstance.getPointPerModelValue()+corePointsSpent)/desiredTotalPoints)*100;
			System.out.println("c " +val);
			if(val<=os.getMinCorePercentage())
				return true;
		}
		else if(unitInstance.getArmyType().equals(os.getRare())){
			float val = ((unitInstance.getPointPerModelValue()+rarePointsSpent)/desiredTotalPoints)*100;
			System.out.println("r" + val);
			if(val>os.getMaxRarePercentage())
				return true;
		}
		else if(unitInstance.getArmyType().equals(os.getSpecial())){
			float val = ((unitInstance.getPointPerModelValue()+specialPointsSpent)/desiredTotalPoints)*100;
			System.out.println("s" + val);
			if(val>os.getMaxSpecialPercentage())
				return true;
		}
		else if(unitInstance.getArmyType().equals(os.getHero())){
			float val = ((unitInstance.getPointPerModelValue()+heroPointsSpent)/desiredTotalPoints)*100;
			System.out.println("h " +val);
			if(val>os.getMaxHeroesPercentage())
				return true;
		}
		else if(unitInstance.getArmyType().equals(os.getLord())){
			float val = ((unitInstance.getPointPerModelValue()+lordPointsSpent)/desiredTotalPoints)*100;
			System.out.println("l " +val);
			if(val>os.getMaxLordsPercentage())
				return true;
		}
		return false;
	}
	
	public boolean checkIfAddingUnitWillBreakLimits(UnitInstance unitInstance){
		if(unitInstance.getArmyType().equals(os.getCore())){
			float val = ((unitInstance.getPointValue()+corePointsSpent)/desiredTotalPoints)*100;
			System.out.println("c2 " +val +" "+ unitInstance);
			if(val<=os.getMinCorePercentage())
				return true;
		}
		else if(unitInstance.getArmyType().equals(os.getRare())){
			float val = ((unitInstance.getPointValue()+rarePointsSpent)/desiredTotalPoints)*100;
			System.out.println("r2 " + val +" "+ unitInstance);
			if(val>os.getMaxRarePercentage())
				return true;
		}
		else if(unitInstance.getArmyType().equals(os.getSpecial())){
			float val = ((unitInstance.getPointValue()+specialPointsSpent)/desiredTotalPoints)*100;
			System.out.println("s2 " + val +" "+ unitInstance);
			if(val>os.getMaxSpecialPercentage())
				return true;
		}
		else if(unitInstance.getArmyType().equals(os.getHero())){
			float val = ((unitInstance.getPointValue()+heroPointsSpent)/desiredTotalPoints)*100;
			System.out.println("h2 " +val +" "+ unitInstance);
			if(val>os.getMaxHeroesPercentage())
				return true;
		}
		else if(unitInstance.getArmyType().equals(os.getLord())){
			float val = ((unitInstance.getPointValue()+lordPointsSpent)/desiredTotalPoints)*100;
			System.out.println("l2 " +val +" "+ unitInstance);
			if(val>os.getMaxLordsPercentage())
				return true;
		}
		if(inArmyUnitCount.containsKey(unitInstance.getName())){
			if((inArmyUnitCount.get(unitInstance.getName())+1)>unitInstance.getMaxInRoster() && unitInstance.getMaxInRoster()>0){
				System.out.println("would increase beyond maxinroster "+ unitInstance);
				return true;
			}
		}
		return false;
	}
	
	
	public List<String> getCores(){
		List<String> ret = new ArrayList<>();
		for (UnitInstance i : cores) {
			ret.add(i.toString());
		}
		return ret;
	}
	
	public List<String> getRares(){
		List<String> ret = new ArrayList<>();
		for (UnitInstance i : rares) {
			ret.add(i.toString());
		}
		return ret;
	}
	
	public List<String> getSpecials(){
		List<String> ret = new ArrayList<>();
		for (UnitInstance i : specials) {
			ret.add(i.toString());
		}
		return ret;
	}
	
	public List<String> getHeroes(){
		List<String> ret = new ArrayList<>();
		for (UnitInstance i : heroes) {
			ret.add(i.toString());
		}
		return ret;
	}
	
	public List<String> getLords(){
		List<String> ret = new ArrayList<>();
		for (UnitInstance i : lords) {
			ret.add(i.toString());
		}
		return ret;
	}
	
	public boolean isAValidSolution(){
		return checkIfDesiredUnitsAreIncluded()&&checkIfPointsAreWithinThresholds()&&checkIfPointsSpentAreCloseToTarget()&&checkIfPointWithinThresholdPointLimit()&&hasACommander();
	}
	
	
	
	
	public float getCorePointsSpent() {
		return corePointsSpent;
	}

	public void setCorePointsSpent(float corePointsSpent) {
		this.corePointsSpent = corePointsSpent;
	}

	public float getRarePointsSpent() {
		return rarePointsSpent;
	}

	public void setRarePointsSpent(float rarePointsSpent) {
		this.rarePointsSpent = rarePointsSpent;
	}

	public float getSpecialPointsSpent() {
		return specialPointsSpent;
	}

	public void setSpecialPointsSpent(float specialPointsSpent) {
		this.specialPointsSpent = specialPointsSpent;
	}

	public float getHeroPointsSpent() {
		return heroPointsSpent;
	}

	public void setHeroPointsSpent(float heroPointsSpent) {
		this.heroPointsSpent = heroPointsSpent;
	}

	public float getLordPointsSpent() {
		return lordPointsSpent;
	}

	public void setLordPointsSpent(float lordPointsSpent) {
		this.lordPointsSpent = lordPointsSpent;
	}

	public float getTotalPointsSpent() {
		return totalPointsSpent;
	}

	public void setTotalPointsSpent(float totalPointsSpent) {
		this.totalPointsSpent = totalPointsSpent;
	}

	public float getDesiredTotalPoints() {
		return desiredTotalPoints;
	}

	public void setDesiredTotalPoints(float desiredTotalPoints) {
		this.desiredTotalPoints = desiredTotalPoints;
	}

	public float getMinimumCostOfUnit() {
		return minimumCostOfUnit;
	}

	public void setMinimumCostOfUnit(float minimumCostOfUnit) {
		this.minimumCostOfUnit = minimumCostOfUnit;
	}

	public float getMinimumCostOfModel() {
		return minimumCostOfModel;
	}

	public void setMinimumCostOfModel(float minimumCostOfModel) {
		this.minimumCostOfModel = minimumCostOfModel;
	}

	public boolean isEnoughToAddAnotherUnit() {
		return enoughToAddAnotherUnit;
	}

	public void setEnoughToAddAnotherUnit(boolean enoughToAddAnotherUnit) {
		this.enoughToAddAnotherUnit = enoughToAddAnotherUnit;
	}

	public boolean isEnoughToIncreaseCountOfUnit() {
		return enoughToIncreaseCountOfUnit;
	}

	public void setEnoughToIncreaseCountOfUnit(boolean enoughToIncreaseCountOfUnit) {
		this.enoughToIncreaseCountOfUnit = enoughToIncreaseCountOfUnit;
	}

	public boolean isCanRemoveGearFromCore() {
		return canRemoveGearFromCore;
	}

	public void setCanRemoveGearFromCore(boolean canRemoveGearFromCore) {
		this.canRemoveGearFromCore = canRemoveGearFromCore;
	}

	public boolean isPossibleToReAddUnits() {
		return possibleToReAddUnits;
	}

	public void setPossibleToReAddUnits(boolean possibleToReAddUnits) {
		this.possibleToReAddUnits = possibleToReAddUnits;
	}

	public boolean isEnoughtoAddUnitButLimited() {
		return enoughtoAddUnitButLimited;
	}

	public void setEnoughtoAddUnitButLimited(boolean enoughtoAddUnitButLimited) {
		this.enoughtoAddUnitButLimited = enoughtoAddUnitButLimited;
	}
}
