package myCBR;

import java.io.File;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Random;

import parserXML.OverviewSpecifics;
import de.dfki.mycbr.core.DefaultCaseBase;
import de.dfki.mycbr.core.Project;
import de.dfki.mycbr.core.casebase.Instance;
import de.dfki.mycbr.core.model.AttributeDesc;
import de.dfki.mycbr.core.model.Concept;

public class AddNewArmyCase {
	
	private Project prj;
	private OverviewSpecifics os;
	private Concept unit;
	private Concept army;
	private Concept profile;
	
	
	public static void main(String[] args) {
		File file = new File("ArmyModel\\ArmyModel.prj");
		
		AddNewArmyCase a = new AddNewArmyCase();
		
		try {
			Project j = new Project(file.getAbsolutePath());
			while(j.isImporting()){}
			a.initOnce(j);
			OverviewSpecifics os = a.getOs();
						
			DefaultCaseBase cb1 = (DefaultCaseBase) j.getCB("Army");
			
			a.addArmy(cb1, os.getDwarfs(), os.getDwarfCB(), a.getRandomHexString(20), 2399);
			
		} catch (Exception e) {
			System.err.println("could not create new army instance");
			e.printStackTrace();
		}
		
	}
		
	public void initOnce(Project prj){
		this.prj = prj;
		this.os = new OverviewSpecifics();
		this.unit = prj.getConceptByID("Unit");
		this.army = prj.getConceptByID("Army");
		this.profile = prj.getConceptByID("Profile");
	}
	
	public OverviewSpecifics getOs(){
		return os;
	}
	
	private String getRandomHexString(int numchars){
        Random r = new Random();
        StringBuffer sb = new StringBuffer();
        while(sb.length() < numchars){
            sb.append(Integer.toHexString(r.nextInt()));
        }
        return sb.toString().substring(0, numchars);
    }
	
	public void addArmy(DefaultCaseBase cb1, String Faction, String factionDB, String suffix, float points){
		DefaultCaseBase factionCB = (DefaultCaseBase) prj.getCB(factionDB);
				
		ArmyInstance armyInstance = new ArmyInstance(prj);
		armyInstance.createArmy(army, Faction+"_"+points+"_"+suffix);
		
		armyInstance.setFaction(Faction);
		armyInstance.setTotalPoints(points);
		
		System.out.println(armyInstance.getArmyInstance());
		
		Collection<Instance> factionCases = factionCB.getCases();
		
		
		HashMap<String,Instance> allFactionUnits = new HashMap<>();
		
		for (Instance instance : factionCases) {
			String pureName = instance.getName().split("_")[0];
			System.out.println(pureName);
			allFactionUnits.put(pureName, instance);
		}
				
		addSpecificRoster(cb1, allFactionUnits,armyInstance);
		cb1.addCase(armyInstance.getArmyInstance());
		
		System.out.println(cb1.getCases().size());
		System.out.println(factionCB.getCases().size());
		System.out.println(prj.getCB("Units").getCases().size());
		System.out.println(prj.getAllInstances().size());
		
		System.out.println(armyInstance.getMultiUnitInstances());
		
//		prj.save();
	}
	
	public void addArmy(DefaultCaseBase cb1, String Faction, float points, List<String> units){
		
		DefaultCaseBase factionCB = (DefaultCaseBase) prj.getCB(os.getCorrectCB(Faction));
		
		ArmyInstance armyInstance = new ArmyInstance(prj);
		armyInstance.createArmy(army, Faction+"_"+points+"_"+getRandomHexString(20));
		
		armyInstance.setFaction(Faction);
		armyInstance.setTotalPoints(points);
		
		System.out.println(armyInstance.getArmyInstance());
		
		Collection<Instance> factionCases = factionCB.getCases();
		
		HashMap<String,Instance> allFactionUnits = new HashMap<>();
		
		for (Instance instance : factionCases) {
			String pureName = instance.getName().split("_")[0];
			System.out.println(pureName);
			allFactionUnits.put(pureName, instance);
		}
		
		DefaultCaseBase unitCB = (DefaultCaseBase) prj.getCB("Units");
		for (int i = 0; i < units.size(); i++) {
			String[] unitData = units.get(i).split("_");
			addSpecificUnit(unitData[0], Integer.parseInt(unitData[1]), Float.parseFloat(unitData[2]), allFactionUnits, armyInstance, unitCB);
		}
//		addSpecificRoster(cb1, allFactionUnits,armyInstance);
		cb1.addCase(armyInstance.getArmyInstance());
		
		System.out.println(cb1.getCases().size());
		System.out.println(factionCB.getCases().size());
		System.out.println(prj.getCB("Units").getCases().size());
		System.out.println(prj.getAllInstances().size());
		
//		prj.save();
	}
	
	public void addSpecificRoster(DefaultCaseBase cb1, HashMap<String,Instance> availUnits, ArmyInstance thisArmy){
		DefaultCaseBase unitCB = (DefaultCaseBase) prj.getCB("Units");
				
		addSpecificUnit("Runesmith", 1, 85, availUnits, thisArmy, unitCB);
		addSpecificUnit("Runesmith", 1, 90, availUnits, thisArmy, unitCB);
		addSpecificUnit("Thane", 1, 193, availUnits, thisArmy, unitCB);
		addSpecificUnit("Quarrellers", 12, 144, availUnits, thisArmy, unitCB);
		addSpecificUnit("Quarrellers", 10, 120, availUnits, thisArmy, unitCB);
		addSpecificUnit("Thunderers", 24, 342, availUnits, thisArmy, unitCB);
		addSpecificUnit("Cannon", 1, 150, availUnits, thisArmy, unitCB);
		addSpecificUnit("Cannon", 1, 145, availUnits, thisArmy, unitCB);
		addSpecificUnit("Grudge Thrower", 1, 155, availUnits, thisArmy, unitCB);
		addSpecificUnit("Gyrocopter", 1, 80, availUnits, thisArmy, unitCB);
		addSpecificUnit("Gyrocopter", 1, 80, availUnits, thisArmy, unitCB);
		addSpecificUnit("Hammerers", 19, 305, availUnits, thisArmy, unitCB);
		addSpecificUnit("Irondrakes", 18, 340, availUnits, thisArmy, unitCB);
		addSpecificUnit("Organ Gun", 1, 170, availUnits, thisArmy, unitCB);
	}
	
	
	//add equipment at some point as well
	public void addSpecificUnit(String nameOfUnit, int modelCount, float totalPoints, HashMap<String,Instance> availUnits, ArmyInstance thisArmy, DefaultCaseBase unitCB){
		
//		Instance pre = availUnits.get(nameOfUnit);
		
		UnitInstance unitToAdd = new UnitInstance(prj);
		unitToAdd.copyFromPreviousUnit(unit, availUnits.get(nameOfUnit), nameOfUnit+"_"+modelCount+"_"+totalPoints+"_"+getRandomHexString(20));
		unitToAdd.setModelCount(modelCount);
		unitToAdd.setPoints(totalPoints);
		
		unitCB.addCase(unitToAdd.getUnitInstance());
		thisArmy.addUnitToMultiUnit(unitToAdd.getUnitInstance());
		
//		System.out.println(unitToAdd.getUnitInstance());
//		System.out.println(thisArmy.getArmyInstance());
		for (AttributeDesc attrdesc : thisArmy.getArmyInstance().getAttributes().keySet()) {
			System.out.print(attrdesc + " "+  thisArmy.getArmyInstance().getAttForDesc(attrdesc) + " ");
		}
		System.out.print("\n");
//		for (AttributeDesc attrdesc : unitToAdd.getUnitInstance().getAttributes().keySet()) {
//			System.out.print(attrdesc + " "+  unitToAdd.getUnitInstance().getAttForDesc(attrdesc).getValueAsString() + " ");
//		}
//		System.out.print("\n");
//		for (AttributeDesc attrdesc : pre.getAttributes().keySet()) {
//			System.out.print(attrdesc + " "+  pre.getAttForDesc(attrdesc).getValueAsString() + " ");
//		}
//		System.out.print("\n");
	}
	
}
