package myCBR;

import java.io.File;
import java.util.Arrays;
import java.util.List;

import parserXML.Catalogue;
import parserXML.Characteristic;
import parserXML.Entry;
import parserXML.Link;
import parserXML.OverviewSpecifics;
import parserXML.Profile;
import parserXML.ReaderMain;
import de.dfki.mycbr.core.DefaultCaseBase;
import de.dfki.mycbr.core.Project;
import de.dfki.mycbr.core.casebase.Instance;
import de.dfki.mycbr.core.model.AttributeDesc;
import de.dfki.mycbr.core.model.Concept;
import de.dfki.mycbr.core.model.ConceptDesc;

public class AddUnitsToCaseBases {
	
	private OverviewSpecifics os;
	private Project prj;
	private Concept profileConcept;
	private Concept unitConcept;
	
	public static void main(String[] args) {
		
		File file = new File("ArmyModel\\ArmyModel.prj");
		
		AddUnitsToCaseBases a = new AddUnitsToCaseBases();
		
		try {
			Project j = new Project(file.getAbsolutePath());
			while(j.isImporting()){}
			a.initOnce(j);
			OverviewSpecifics os = a.getOs();
			
//			System.out.println(j.getCaseBases());
			
			ReaderMain rm = new ReaderMain();
			rm.initiateReader();
			
			for (Catalogue cat : rm.getCatalogues()) {
				cat.initMap();
				if(cat.getName().contains(os.getDwarfs())){
					DefaultCaseBase cb1 = (DefaultCaseBase) j.getCB(os.getDwarfCB());
					System.out.println(cb1.getCases());
					a.addUnits(cat,cb1);
					for (Instance i : cb1.getCases()) {
						System.out.println(i + " " + i.getAttributes());
					}
				}
				if(cat.getName().contains(os.getHighelves())){
					DefaultCaseBase cb1 = (DefaultCaseBase) j.getCB(os.getHighelvesCB());
					System.out.println(cb1.getCases());
					a.addUnits(cat,cb1);
					for (Instance i : cb1.getCases()) {
						System.out.println(i + " " + i.getAttributes());
					}
				}
				//skaven
//				else if(cat.getName().contains(os.getSkaven())){
//					DefaultCaseBase cb1 = (DefaultCaseBase) j.getCB(os.getSkavenCB());
//					a.addUnits(cat,cb1);
//					for (Instance i : cb1.getCases()) {
//						System.out.println(i + "   " + i.getAttributes());
//					}
//				}
			}
			
			
			System.out.println("Dwarf units : " + j.getCB(os.getDwarfCB()).getCases().size());
			System.out.println("High elf units : " + j.getCB(os.getHighelvesCB()).getCases().size());
//			System.out.println("Skaven units : " + j.getCB(os.getSkavenCB()).getCases().size());
			System.out.println("Total profiles : " + j.getCB("Profiles").getCases().size());
			
			Concept unit = j.getConceptByID("Unit");
			
			for (Instance k : j.getCB("Profiles").getCases()) {
				boolean used = false;
				for (Instance i : j.getCB(os.getHighelvesCB()).getCases()) {
					ConceptDesc unitProfile = (ConceptDesc) unit.getAttributeDesc("UnitProfile");
					if(i.getAttForDesc(unitProfile)==k){
						used = true;
						break;
					}
					
				}
				for (Instance i : j.getCB(os.getDwarfCB()).getCases()) {
					ConceptDesc unitProfile = (ConceptDesc) unit.getAttributeDesc("UnitProfile");
					if(i.getAttForDesc(unitProfile)==k){
						used = true;
						break;
					}
					
				}
				if(!used)
					System.out.println(k);
			}
			
			
			System.out.println(j.getAllInstances().size());
			
			j.save(); //save is dangerous. 
		} catch (Exception e) {
			System.err.println("could not import and export all the shizzle");
			e.printStackTrace();
		}
		
	}
	
	public void initOnce(Project j){
		os = new OverviewSpecifics();
		prj = j;
		profileConcept = prj.getConceptByID("Profile");
		unitConcept = prj.getConceptByID("Unit");
	}
	
	//TODO, add sub units. both units and upgrades such as musicians etc
	
	public void addUnits(Catalogue cat, DefaultCaseBase cb){
		for (Entry e : cat.getEntries()) {
			if(os.isArmyUnit(e.getCategoryId())){
				UnitInstance unitTobeAdded = new UnitInstance(prj);
				if(e.getPoints()==0){
					Entry e2 = getSubEntryRec(cat,e);
					if(e2 == null)
						System.err.println("something went wrong with getting a subEntry when adding units");
					else
						addUnitInfo(cat, e2, unitTobeAdded, e);
				}
				else{
					addUnitInfo(cat, e, unitTobeAdded,e);
				}
				cb.addCase(unitTobeAdded.getUnitInstance());
			}
		}
	}
	
	public Entry getSubEntryRec(Catalogue cat, Entry entry){
		List<Entry> entries = entry.getEntries();
		for (Entry entry2 : entries) {
			if(entry.getName().contains(entry2.getName())){
				return entry2;
			}
			else{
				if(!entry2.getType().equals("model"))
					System.out.println("\t" + entry2.getName() + " subUnit. not handled yet");
				String[] substr = entry2.getName().split(" ");
				System.out.println("\t" + Arrays.toString(substr));
				for (String string : substr) {
					if(entry.getName().contains(string) && entry2.getType().equals("model"))
						return entry2;
				}
			}
		}
		if(entries.size() == 1){
			System.err.println("Only one subEntry - has to be the correct one");
			return entries.get(0);
		}
			
		return null;
	}
	
	
	
	//Missing entrygroups as well
	public void addUnitInfo(Catalogue cat, Entry entry, UnitInstance unitInstance, Entry topEntry){
		String entryName = topEntry.getName();
		String unitCategory = topEntry.getCategoryId();
		String entryId = topEntry.getId();
		
		List<Profile> profiles = entry.getProfiles();
		
		unitInstance.createUnit(unitConcept, entryName+"_"+(entry.getMinSelections()>0?entry.getMinSelections():1)+"_"+entry.getPoints()+"_"+unitCategory(unitCategory)+"_"+entryId);
		System.out.println("u " + unitInstance.getUnitInstance().getName());
		
		unitInstance.setModelCount(entry.getMinSelections()>0?entry.getMinSelections():1);
		unitInstance.setName(entryName);
		unitInstance.setArmyType(unitCategory(unitCategory));
		unitInstance.setPoints(entry.getPoints()*unitInstance.getModelCount());
		unitInstance.setCostPerModel(entry.getPoints());
		unitInstance.setMinSelections(entry.getMinSelections());
		unitInstance.setMaxInRoster(topEntry.getMaxInRoster());
		
		if(profiles.isEmpty()){
			Entry e2 = getSubEntryRec(cat, entry);
			profiles = e2.getProfiles();
		}
		
		
		
		for (Profile profile : profiles) {
			if(profile.getProfileTypeId().equals(os.getModelId())){
				System.err.println(profile.getName());
				boolean sameModel = false;
				if(unitInstance.getName().contains(profile.getName()))
						sameModel = true;
				else{
					String[] profileNameStr = profile.getName().split(" ");
					
					for (String string : profileNameStr) {
						System.err.println(string);
						if(unitInstance.getName().contains(string))
							sameModel = true;
						if(string.toLowerCase().equals("crew")){ //really ugly hack...
							System.err.println("Ugly hack for crew profile where multiple profiles");
							sameModel = false;
						}
					}
					if(!sameModel && profileNameStr.length == 1){
						System.err.println("Ugly hack for man to men");
						String replaceman = profileNameStr[0].replace("man", "men");
						if(replaceman.equals(unitInstance.getName())){
							System.err.println("could change from man to men");
							sameModel = true;
						}
					}
				}
				if(sameModel)
					iterateProfile(profile, unitInstance);
				else
					System.out.println("some other model profile - handle later");
			}
			else
				iterateProfile(profile, unitInstance);
		}
		
		List<Link> links = entry.getLinks();
		for (Link link : links) {
			String target = link.getTargetId();
			String type = link.getLinkType();
			if(type.equals("profile")){
				System.err.println("p "+ cat.getSharedProfilesMap().get(target).getName());
				iterateProfile(cat.getSharedProfilesMap().get(target), unitInstance);
			}
			else if(type.equals("rule")){
				System.err.println("r "+ cat.getSharedRulesMap().get(target).getName());
			}
		}
		
		//topentry links
		List<Link> TopEntrylinks = topEntry.getLinks();
		for (Link link : TopEntrylinks) {
			String target = link.getTargetId();
			String type = link.getLinkType();
			if(type.equals("profile")){
				System.err.println("p "+ cat.getSharedProfilesMap().get(target).getName());
				if(!cat.getSharedProfilesMap().get(target).getProfileTypeId().equals(os.getModelId()))
					iterateProfile(cat.getSharedProfilesMap().get(target), unitInstance);
				else
					System.err.println("\t unhandled profile for a model not used in the cb");
			}
			else if(type.equals("rule")){
				System.err.println("r "+ cat.getSharedRulesMap().get(target).getName());
			}
		}
		
		
		System.out.println(unitInstance.getUnitInstance().getAttributes());
		System.out.println();
		
		
	}
	

	//TODO proper dice values for D6.
	public float valueOfStringCharacteristics(String in){
		String val = in;
		System.err.print(val+ " ");
		if(val == null || val.equals("") || val.equals("-") || val.equals("*")) //
			val = new String("0");
		if(val.contains("D6")){
			val = val.substring(0, 1);
			val = ""+Float.parseFloat(val)*3; //avg of 6 is 3. this is not sound for more than 1 die, but have to start somewhere
		}
		else
			val = val.replaceAll("[^\\d.]", "");
		System.err.print(val + "\n");
		return Float.parseFloat(val);
	}
	
	public float valueOfStringCharacteristicsWeaponRange(String input){
		String val = input;
		if(val == null || val.equals("") || val.equals("-"))
			val = new String("0");
		val = val.replaceAll("[^\\d.]", "");
		if(val.isEmpty())
			val += "0";
		return Float.parseFloat(val);
	}
	
	public String unitTypeProfileCharacteristic(String input){
		if(input.contains(os.getMonsterousInfantry()))
			return os.getMonsterousInfantry();
		else if(input.contains(os.getMonsterousCavalry()))
			return os.getMonsterousCavalry();
		else if(input.contains(os.getMonster()))
			return os.getMonster();
		else if(input.contains(os.getInfantry()))
			return os.getInfantry();
		else if(input.contains(os.getCavalry()))
			return os.getCavalry();
		else if(input.contains(os.getChariot()))
			return os.getChariot();
		else if(input.contains(os.getWarBeast()))
			return os.getWarBeast();
		else if(input.contains(os.getWarMachine()))
			return os.getWarMachine();
		else if(input.contains(os.getSwarm()))
			return os.getSwarm();
		else
			return os.getUnspecified();
	}
	
	public String unitCategory(String input){
		if(input.equals(os.getCoresId()))
			return os.getCore();
		else if(input.equals(os.getRareId()))
			return os.getRare();
		else if(input.equals(os.getSpecialId()))
			return os.getSpecial();
		else if(input.equals(os.getLordsId()))
			return os.getLord();
		else if(input.equals(os.getHeroesId()))
			return os.getHero();
		System.err.println("something is wrong and null is unit category");
		return null;
	}
	
	public void iterateProfile(Profile profile, UnitInstance unitInstance){
		boolean magician = unitInstance.getMagician();
		boolean ranged = unitInstance.getRanged();
		List<Characteristic> characteristics = profile.getCharacteristics();
		DefaultCaseBase cb2 = (DefaultCaseBase) prj.getCB("Profiles");
		if(profile.getProfileTypeId().equals(os.getModelId())){		
			ProfileInstance profileToBeAdded = new ProfileInstance(prj);
			profileToBeAdded.createProfile(profileConcept, profile.getName()+"_Profile_"+profile.getId());
			for (Characteristic characteristic : characteristics) {
				if(characteristic.getCharacteristicId().equals(os.getMagicResistId())){
					float valNum = valueOfStringCharacteristics(characteristic.getValue());
					profileToBeAdded.setMR(valNum);						
				}
				else if(characteristic.getCharacteristicId().equals(os.getMovementId())){
					float valNum = valueOfStringCharacteristics(characteristic.getValue());
					profileToBeAdded.setM(valNum);						
				}
				else if(characteristic.getCharacteristicId().equals(os.getWeaponSkillId())){
					float valNum = valueOfStringCharacteristics(characteristic.getValue());
					profileToBeAdded.setWS(valNum);						
				}
				else if(characteristic.getCharacteristicId().equals(os.getBsId())){
					float valNum = valueOfStringCharacteristics(characteristic.getValue());
					profileToBeAdded.setBS(valNum);			
				}
				else if(characteristic.getCharacteristicId().equals(os.getStrengthUnitId())){
					float valNum = valueOfStringCharacteristics(characteristic.getValue());
					profileToBeAdded.setS(valNum);			
				}
				else if(characteristic.getCharacteristicId().equals(os.getToughnessId())){
					float valNum = valueOfStringCharacteristics(characteristic.getValue());
					profileToBeAdded.setT(valNum);			
				}
				else if(characteristic.getCharacteristicId().equals(os.getWoundsId())){
					float valNum = valueOfStringCharacteristics(characteristic.getValue());
					profileToBeAdded.setW(valNum);			
				}
				else if(characteristic.getCharacteristicId().equals(os.getInitiativeId())){
					float valNum = valueOfStringCharacteristics(characteristic.getValue());
					profileToBeAdded.setI(valNum);
				}
				else if(characteristic.getCharacteristicId().equals(os.getAttacksId())){
					float valNum = valueOfStringCharacteristics(characteristic.getValue());
					profileToBeAdded.setA(valNum);
				}
				else if(characteristic.getCharacteristicId().equals(os.getLeadershipId())){
					float valNum = valueOfStringCharacteristics(characteristic.getValue());
					profileToBeAdded.setLd(valNum);
				}
				else if(characteristic.getCharacteristicId().equals(os.getArmorSaveId())){
					float valNum = valueOfStringCharacteristics(characteristic.getValue());
					profileToBeAdded.setArmorSave(valNum);
				}
				else if(characteristic.getCharacteristicId().equals(os.getWardSaveId())){
					float valNum = valueOfStringCharacteristics(characteristic.getValue());
					profileToBeAdded.setWardSave(valNum);
				}
				else if(characteristic.getCharacteristicId().equals(os.getTypeUnitId())){
					String unitType = unitTypeProfileCharacteristic(characteristic.getValue());
					profileToBeAdded.setUnitType(unitType);
				}
			}
			profileToBeAdded.setName(profile.getName());
			System.out.print("p "+ profileToBeAdded.getProfileInstance() + "\n");
			for (AttributeDesc a : profileToBeAdded.getProfileInstance().getAttributes().keySet()) {
				System.out.print(a + " " + profileToBeAdded.getProfileInstance().getAttForDesc(a).getValueAsString() + ", ");
			}
			System.out.print("\n");
			unitInstance.setProfile(profileToBeAdded.getProfileInstance());
			cb2.addCase(profileToBeAdded.getProfileInstance());
		}
		//TODO add weapon in equipment
		else if(profile.getProfileTypeId().equals(os.getWeaponId())){
			System.err.println(profile.getName());
			for (Characteristic characteristic : characteristics) {
				if(characteristic.getCharacteristicId().equals(os.getRangeId())){
					float value = valueOfStringCharacteristicsWeaponRange(characteristic.getValue());
					if(value>0)
						ranged = true;
					System.err.println("range " + value + " " + ranged);
				}
			}
		}
		//TODO add armor in equipment
		else if(profile.getProfileTypeId().equals(os.getArmorId())){
			System.out.print("armor " + profile.getName());
			for (Characteristic characteristic : characteristics) {
				System.out.print(characteristic.getName() + " " + characteristic.getValue() +" ");
			}
			System.out.print("\n");
		}
		//TODO add to equipment
		else if(profile.getProfileTypeId().equals(os.getArcaneEnchantedItemId())){
			for (Characteristic characteristic : characteristics) {
				System.out.print(characteristic.getName() + " " + characteristic.getValue() +" ");
			}
		}
		//TODO add to equipment
		else if(profile.getProfileTypeId().equals(os.getTalismanId())){
			for (Characteristic characteristic : characteristics) {
				System.out.print(characteristic.getName() + " " + characteristic.getValue() +" ");
			}
		}
		//TODO add to equipment
		else if(profile.getProfileTypeId().equals(os.getMagicStandardId())){
			for (Characteristic characteristic : characteristics) {
				System.out.print(characteristic.getName() + " " + characteristic.getValue() +" ");
			}
		}
		unitInstance.setRanged(ranged);
		
	}

	public OverviewSpecifics getOs() {
		return os;
	}
}
