package myCBR;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import de.dfki.mycbr.core.Project;
import de.dfki.mycbr.core.casebase.Attribute;
import de.dfki.mycbr.core.casebase.Instance;
import de.dfki.mycbr.core.casebase.MultipleAttribute;
import de.dfki.mycbr.core.model.AttributeDesc;
import de.dfki.mycbr.core.model.Concept;
import de.dfki.mycbr.core.model.ConceptDesc;
import de.dfki.mycbr.core.model.FloatDesc;
import de.dfki.mycbr.core.model.StringDesc;
import de.dfki.mycbr.core.model.SymbolDesc;

public class ArmyInstance {
	private Instance thisArmyInstance;
	private Concept army;
	private Concept unit;
	private Project prj;
	private MultipleAttribute<ConceptDesc> multiUnits;
	private MultipleAttribute<ConceptDesc> multiUnwantedUnits;
	
	
	
	
	public ArmyInstance(Project project){
		this.prj = project;
		this.unit = prj.getConceptByID("Unit");
	}
	
	public void initEmptyArmy(){
		EmptyInstance ei = new EmptyInstance();
		ei.reset(thisArmyInstance, prj);
	}
	
	public void createArmy(Concept army, String name){
		try {
			this.army = army;
			thisArmyInstance = army.addInstance(name);
			initEmptyArmy();
			initMultiUnit();
//			initMultiUnwantedUnit();
		} catch (Exception e) {
			System.err.println("Could not add army instance " + name);
			e.printStackTrace();
		}
	}
	
	
	//TODO most likely not needed
	public void usePreviousArmy(Concept army, Instance armyInstance){
		thisArmyInstance = armyInstance;
		this.army = army;
//		initEmptyArmy();
	}
	
	
	public void copyArmyInstance(Concept army, Instance armyInstance, String name){
		try {
			this.army = army;
			thisArmyInstance = army.addInstance(name);
			for (AttributeDesc attr : armyInstance.getAttributes().keySet()) {
				if(attr.getName().equals("Units")){
					initMultiUnit();
					if(armyInstance.getAttForDesc(attr).toString().contains(";")){
						MultipleAttribute<AttributeDesc> au = (MultipleAttribute<AttributeDesc>) armyInstance.getAttForDesc(attr);
						for (Attribute attr2 : au.getValues()) {
							addUnitToMultiUnit((Instance)attr2);
						}
					}
				}
//				else if(attr.getName().equals("UnwantedUnits")){
//					initMultiUnwantedUnit();
//					if(armyInstance.getAttForDesc(attr).toString().contains(";")){
//						MultipleAttribute<AttributeDesc> au = (MultipleAttribute<AttributeDesc>) armyInstance.getAttForDesc(attr);
//						for (Attribute attr2 : au.getValues()) {
//							addUnitToMultiUnit((Instance)attr2);
//						}
//					}
//				}
				else
					thisArmyInstance.addAttribute(attr, armyInstance.getAttForDesc(attr));
			}
		} catch (Exception e) {
			System.err.println("could not copy army");
			e.printStackTrace();
		}		
	}
	
	public void setFactionId(String factionId){
		StringDesc factionIdDesc = (StringDesc) army.getAttributeDesc("FactionId");
		try {
			thisArmyInstance.addAttribute(factionIdDesc, factionId);
		} catch (ParseException e) {
			System.err.println("Could not set string for army factionId " + factionId);
			e.printStackTrace();
		}
	}
	
	public void initMultiUnit(){
		ConceptDesc unitDesc = (ConceptDesc) army.getAttributeDesc("Units");
		multiUnits = new MultipleAttribute<ConceptDesc>(unitDesc, new LinkedList<Attribute>());
		thisArmyInstance.addAttribute(unitDesc, multiUnits);
	}
	
	public void addUnitToMultiUnit(Instance unit){
		multiUnits.addValue(unit);
	}
	
	public void removeUnitFromMultiUnit(Instance unit){
		multiUnits.removeValue(unit);
	}
	
	public void initMultiUnitExisting(){
		ConceptDesc unitDesc = (ConceptDesc) army.getAttributeDesc("Units");
		multiUnits = (MultipleAttribute<ConceptDesc>) thisArmyInstance.getAttForDesc(unitDesc);
	}
	
	public List<UnitInstance> getMultiUnitInstances(){
		List<UnitInstance>  ret = new ArrayList<UnitInstance>();
		for (Attribute iter : multiUnits.getValues()) {
			UnitInstance t = new UnitInstance(prj);
			t.usePreviousUnit(unit, (Instance) iter);
			t.initializeUnitValues();
			ret.add(t);
		}
		
		return ret;
	}
	
	/**
	 * currently not working as intended so not used. (negative weights not working producing the proper results)
	 */
	public void initMultiUnwantedUnit(){
		ConceptDesc unitDesc = (ConceptDesc) army.getAttributeDesc("UnwantedUnits");
		multiUnwantedUnits = new MultipleAttribute<ConceptDesc>(unitDesc, new LinkedList<Attribute>());
		thisArmyInstance.addAttribute(unitDesc, multiUnwantedUnits);
	}
	
	public void addUnitToMultiUnwatnedUnit(Instance unit){
		multiUnwantedUnits.addValue(unit);
	}
	
	public void removeUnitFromMultiUnwatnedUnit(Instance unit){
		multiUnwantedUnits.removeValue(unit);
	}
	
	
	
	public void setTotalPoints(float points){
		FloatDesc totalPointDesc = (FloatDesc) army.getAttributeDesc("TotalPoints");
		try {
			thisArmyInstance.addAttribute(totalPointDesc, points);
		} catch (ParseException e) {
			System.err.println("could not add totalPoints to army");
			e.printStackTrace();
		}
	}
	
	public float getTotalPoints(){
		FloatDesc totalPointDesc = (FloatDesc) army.getAttributeDesc("TotalPoints");
		return Float.parseFloat(thisArmyInstance.getAttForDesc(totalPointDesc).getValueAsString());
	}
	
	public void setFaction(String faction){
		SymbolDesc factionDesc = (SymbolDesc) army.getAttributeDesc("Faction");
		thisArmyInstance.addAttribute(factionDesc, factionDesc.getAttribute(faction));
	}
	
	public String getFaction(){
		SymbolDesc factionDesc = (SymbolDesc) army.getAttributeDesc("Faction");
		return thisArmyInstance.getValueAsString();
	}
	
	public Instance getArmyInstance(){
		return thisArmyInstance;
	}
}
