package myCBR;

import java.text.ParseException;
import java.util.HashMap;

import de.dfki.mycbr.core.Project;
import de.dfki.mycbr.core.casebase.Attribute;
import de.dfki.mycbr.core.casebase.Instance;
import de.dfki.mycbr.core.model.AttributeDesc;
import de.dfki.mycbr.core.model.BooleanDesc;
import de.dfki.mycbr.core.model.Concept;
import de.dfki.mycbr.core.model.ConceptDesc;
import de.dfki.mycbr.core.model.FloatDesc;
import de.dfki.mycbr.core.model.IntegerDesc;
import de.dfki.mycbr.core.model.StringDesc;
import de.dfki.mycbr.core.model.SymbolDesc;

public class UnitInstance {
	private Instance thisUnitInstance;
	private Concept unit;
	private Project prj;
	
	private int modelCount;
	private float point;
	private float pointPerModel;
	private float surplussPoints;
	
	
	
	public UnitInstance(Project j){
		prj = j;
	}
	
	public void createUnit(Concept unit, String name) {
		try {
			thisUnitInstance = unit.addInstance(name);
			this.unit = unit;
			initEmptyUnit();
		} catch (Exception e) {
			System.err.println("Something went wrong with unit creation");
			e.printStackTrace();
		}
	}
	
	
	/**
	 * for use with query for example. remember to initEmptyUnit() if using a previous unit with all empty fields (query)
	 * @param unit
	 * @param unitInstance
	 */
	public void usePreviousUnit(Concept unit, Instance unitInstance){
		thisUnitInstance = unitInstance;
		this.unit = unit;
//		initEmptyUnit();
	}
	
	public void copyFromPreviousUnit(Concept unit, Instance unitInstance, String name){
		try {
			this.unit = unit;
			thisUnitInstance = unit.addInstance(name);
			for (AttributeDesc attr : unitInstance.getAttributes().keySet()) {
				thisUnitInstance.addAttribute(attr, unitInstance.getAttForDesc(attr));
			}
		} catch (Exception e) {
			System.err.println("Could not create new unit from old unit");
			e.printStackTrace();
		}
	}
	
	public void InitFromOtherInstance(Instance unitInstance){
		for (AttributeDesc attr : unitInstance.getAttributes().keySet()) {
			thisUnitInstance.addAttribute(attr, unitInstance.getAttForDesc(attr));
		}
	}
	
	public void initEmptyUnit(){
		EmptyInstance ei = new EmptyInstance();
		ei.reset(thisUnitInstance, prj);
	}
	
	
	public void setProfile(Instance profile){
		ConceptDesc unitProfile = (ConceptDesc) unit.getAttributeDesc("UnitProfile");
		thisUnitInstance.addAttribute(unitProfile, profile);
	}
	
	public Instance getProfile(){
		ConceptDesc unitProfile = (ConceptDesc) unit.getAttributeDesc("UnitProfile");
		return (Instance) thisUnitInstance.getAttForDesc(unitProfile);
	}
	
	public void setModelCount(int count){
		IntegerDesc modelCount = (IntegerDesc) unit.getAttributeDesc("ModelCount");
		try {
			thisUnitInstance.addAttribute(modelCount, modelCount.getAttribute(count));
		} catch (ParseException e) {
			System.err.println("could not set integer value for modelcount");
			e.printStackTrace();
		}
	}
	
	public int getModelCount(){
		IntegerDesc modelCount = (IntegerDesc) unit.getAttributeDesc("ModelCount");
		return Integer.parseInt(thisUnitInstance.getAttForDesc(modelCount).getValueAsString());
	}
	
	public void setPoints(float points){
		FloatDesc pointCount = (FloatDesc) unit.getAttributeDesc("Point");
		try {
			thisUnitInstance.addAttribute(pointCount, pointCount.getAttribute(points));
		} catch (ParseException e) {
			System.err.println("could not set float value for points");
			e.printStackTrace();
		}
	}
	
	public float getPoints(){
		FloatDesc pointCount = (FloatDesc) unit.getAttributeDesc("Point");
		return Float.parseFloat(thisUnitInstance.getAttForDesc(pointCount).getValueAsString());
	}
	
	
	public void setCostPerModel(float points){
		FloatDesc pointCount = (FloatDesc) unit.getAttributeDesc("CostPerModel");
		try {
			thisUnitInstance.addAttribute(pointCount, pointCount.getAttribute(points));
		} catch (ParseException e) {
			System.err.println("could not set float value for costpermodel");
			e.printStackTrace();
		}
	}
	
	public float getCostPerModel(){
		FloatDesc pointCount = (FloatDesc) unit.getAttributeDesc("CostPerModel");
		return Float.parseFloat(thisUnitInstance.getAttForDesc(pointCount).getValueAsString());
	}
	
	public void setMaxInRoster(int count){
		IntegerDesc modelmaxInRoster = (IntegerDesc) unit.getAttributeDesc("maxInRoster");
		try {
			thisUnitInstance.addAttribute(modelmaxInRoster, modelmaxInRoster.getAttribute(count));
		} catch (ParseException e) {
			System.err.println("could not set integer value for maxInRoster");
			e.printStackTrace();
		}
	}
	
	public int getMaxInRoster(){
		IntegerDesc modelmaxInRoster = (IntegerDesc) unit.getAttributeDesc("maxInRoster");
		return Integer.parseInt(thisUnitInstance.getAttForDesc(modelmaxInRoster).getValueAsString());
	}
	
	
	public void setMinSelections(int count){
		IntegerDesc modelMinSelections = (IntegerDesc) unit.getAttributeDesc("minSelections");
		try {
			thisUnitInstance.addAttribute(modelMinSelections, modelMinSelections.getAttribute(count));
		} catch (ParseException e) {
			System.err.println("could not set integer value for minSelections");
			e.printStackTrace();
		}
	}
	
	public int getMinSelections(){
		IntegerDesc modelMinSelections = (IntegerDesc) unit.getAttributeDesc("minSelections");
		return Integer.parseInt(thisUnitInstance.getAttForDesc(modelMinSelections).getValueAsString());
	}
	
	
	public void setEquipment(Instance equipment){
		//TODO Equipment not currently working. (can't put a value on strings.) future work stuff honestly.
		ConceptDesc unitEquipment = (ConceptDesc) unit.getAttributeDesc("Equipment");
		thisUnitInstance.addAttribute(unitEquipment, equipment);
	}
	
	public void setName(String name){
		//TODO might be redundant
		StringDesc unitName = (StringDesc) unit.getAttributeDesc("Name");
		try {
			thisUnitInstance.addAttribute(unitName, name);
		} catch (ParseException e) {
			System.err.println("could not set string for name");
			e.printStackTrace();
		}
	}
	
	public String getName(){
		StringDesc unitName = (StringDesc) unit.getAttributeDesc("Name");
		return thisUnitInstance.getAttForDesc(unitName).getValueAsString();
	}
	
	public void setRanged(boolean val){
		BooleanDesc unitRanged = (BooleanDesc) unit.getAttributeDesc("Ranged");
		try {
			thisUnitInstance.addAttribute(unitRanged, val);
		} catch (ParseException e) {
			System.err.println("could not set boolean value for ranged");
			e.printStackTrace();
		}
	}
	
	public boolean getRanged(){
		BooleanDesc unitRanged = (BooleanDesc) unit.getAttributeDesc("Ranged");
		return Boolean.parseBoolean(thisUnitInstance.getAttForDesc(unitRanged).getValueAsString());
	}
	
	public void setArmyType(String category){
		SymbolDesc armyType = (SymbolDesc) unit.getAttributeDesc("ArmyType");
		thisUnitInstance.addAttribute(armyType, armyType.getAttribute(category));
	}
	
	public String getArmyType(){
		SymbolDesc armyType = (SymbolDesc) unit.getAttributeDesc("ArmyType");
		return thisUnitInstance.getAttForDesc(armyType).getValueAsString();
	}
	
	public void setMagician(boolean val){
		BooleanDesc unitMagician = (BooleanDesc) unit.getAttributeDesc("Magician");
		try {
			thisUnitInstance.addAttribute(unitMagician, val);
		} catch (ParseException e) {
			System.err.println("could not set boolean value for magician");
			e.printStackTrace();
		}
	}
	
	public boolean getMagician(){
		BooleanDesc unitMagician = (BooleanDesc) unit.getAttributeDesc("Magician");
		return Boolean.parseBoolean(thisUnitInstance.getAttForDesc(unitMagician).getValueAsString());
	}
	
	public Instance getUnitInstance(){
		return thisUnitInstance;
	}
	
	public float getPointValue() {
		return point;
	}

	public void setPointValue(float point) {
		this.point = point;
	}

	public float getPointPerModelValue() {
		return pointPerModel;
	}

	public void setPointPerModelValue(float pointPerModel) {
		this.pointPerModel = pointPerModel;
	}
	
	public int getModelCountValue(){
		return modelCount;
	}
	
	public void setModelCountValue(int val){
		this.modelCount = val;
	}

	public float getSurplussPoints() {
		return surplussPoints;
	}

	public void setSurplussPoints(float surplussPoints) {
		this.surplussPoints = surplussPoints;
	}
	
	public void initializeUnitValues(){
		setPointValue(getPoints());
		setModelCountValue(getModelCount());
		setPointPerModelValue(getCostPerModel());
		setSurplussPoints((getPointValue() - (getModelCountValue()*getPointPerModelValue())));
		
//		System.out.println("cost:  " + getPointValue()  + " count " + getModelCountValue() + " permodel " + getPointPerModelValue() + " surpluss " + getSurplussPoints());
	}
	
	public String toString(){
		return getName() + "_" + getModelCountValue() + "_" + getPointValue();
	}
}
