package parserXML;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import javax.xml.stream.XMLEventReader;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.events.Attribute;
import javax.xml.stream.events.EndElement;
import javax.xml.stream.events.StartElement;
import javax.xml.stream.events.XMLEvent;

public class Catalogue {
	private String id;
	private String revision;
	private String gameSystemId;
	private String gameSystemRevision;
	private String battleScribeVersion;
	private String name;
	private String books;
	private String authorName;
	private String authorContact;
	private String authorUrl;
	
	
	
	private List<String> attributes;
	private List<String> attributeValues;
	
	private List<String> fields;
	
	private List<Entry> entries;
	private List<Rule> rules;
	private List<Link> links;
	private List<Entry> sharedEntries;
	private List<EntryGroup> sharedEntryGroups;
	private List<Rule> sharedRules;
	private List<Profile> sharedProfiles;
	
	private Map<String, Rule> sharedRulesMap;
	private Map<String, Profile> sharedProfilesMap;
	private Map<String, Entry> entriesMap;
	
	public Catalogue(XMLEventReader events, XMLEvent currentEvent) throws XMLStreamException{
		attributes = new ArrayList<String>();
		attributeValues = new ArrayList<String>();
		fields = new ArrayList<String>();
		
		XMLEvent event = currentEvent;
//		System.out.println("start -- catalogue");
		if(event.isStartElement()){
			StartElement elm = event.asStartElement();
			if(elm.getName().getLocalPart().equals("catalogue")){
				Iterator<Attribute> attr = elm.getAttributes();
				while(attr.hasNext()){
					Attribute Tempattribute = attr.next();
					
					if(Tempattribute.getName().getLocalPart().equals("id")){
						id = Tempattribute.getValue();
					}
					else if(Tempattribute.getName().getLocalPart().equals("revision")){
						revision = Tempattribute.getValue();
					}
					else if(Tempattribute.getName().getLocalPart().equals("gameSystemId")){
						gameSystemId = Tempattribute.getValue();
					}
					else if(Tempattribute.getName().getLocalPart().equals("gameSystemRevision")){
						gameSystemRevision = Tempattribute.getValue();
					}
					else if(Tempattribute.getName().getLocalPart().equals("battleScribeVersion")){
						battleScribeVersion = Tempattribute.getValue();
					}
					else if(Tempattribute.getName().getLocalPart().equals("name")){
						name = Tempattribute.getValue();
					}
					else if(Tempattribute.getName().getLocalPart().equals("books")){
						books = Tempattribute.getValue();
					}
					else if(Tempattribute.getName().getLocalPart().equals("authorName")){
						authorName = Tempattribute.getValue();
					}
					else if(Tempattribute.getName().getLocalPart().equals("authorContact")){
						authorContact = Tempattribute.getValue();
					}
					else if(Tempattribute.getName().getLocalPart().equals("authorUrl")){
						authorUrl = Tempattribute.getValue();
					}
					
					attributes.add(Tempattribute.getName().getLocalPart());
					attributeValues.add(Tempattribute.getValue());
				}
			}
		}
		
//		System.out.println(attributes);
//		System.out.println(attributeValues);
//		System.out.println(gameSystemRevision + " "+ books + " "+ gameSystemId + " "+ authorName + " "+ authorUrl + " "+ battleScribeVersion + " "+ name + " "+ id + " "+ authorContact + " "+ revision);
		
		Entries tempEntries = null;
		Rules tempRules = null;
		Links tempLinks = null;
		SharedEntries tempSharedEntries = null;
		SharedEntryGroups tempSharedEntryGroups = null;
		SharedRules tempSharedRules = null;
		SharedProfiles tempSharedProfiles = null;
		
		while(events.hasNext()){
			event = events.nextEvent();
			if (event.isStartElement()) {
				StartElement startElm = event.asStartElement();
				fields.add(startElm.getName().getLocalPart());
				if(startElm.getName().getLocalPart().equals("entries")){
					tempEntries = new Entries(events, event);
					entries = tempEntries.getEntries();
				}
				else if(startElm.getName().getLocalPart().equals("rules")){
					tempRules = new Rules(events,event);
					rules = tempRules.getRules();
				}
				else if(startElm.getName().getLocalPart().equals("links")){
					tempLinks = new Links(events,event);
					links = tempLinks.getLinks();
				}
				else if(startElm.getName().getLocalPart().equals("sharedEntries")){
					tempSharedEntries = new SharedEntries(events, event);
					sharedEntries = tempSharedEntries.getEntries();
				}
				else if(startElm.getName().getLocalPart().equals("sharedEntryGroups")){
					tempSharedEntryGroups = new SharedEntryGroups(events,event);
					sharedEntryGroups = tempSharedEntryGroups.getEntryGroups();
				}
				else if(startElm.getName().getLocalPart().equals("sharedRules")){
					tempSharedRules = new SharedRules(events,event);
					sharedRules = tempSharedRules.getRules();
				}
				else if(startElm.getName().getLocalPart().equals("sharedProfiles")){
					tempSharedProfiles = new SharedProfiles(events,event);
					sharedProfiles = tempSharedProfiles.getProfiles();
				}				
			}
			if(event.isEndElement()){
				EndElement endElm = event.asEndElement();
				if(endElm.getName().getLocalPart().equals("catalogue")){
//					System.out.print(fields);
//					System.out.print("end -- catalogue \n");
					break;
				}
			}
		}
	}
	
	
	public void initMap(){
		sharedProfilesMap = new TreeMap<String,Profile>();
		for (Profile p : getSharedProfiles()) {
			sharedProfilesMap.put(p.getId(), p);
		}
		sharedRulesMap = new TreeMap<String,Rule>();
		for (Rule r : getSharedRules()) {
			sharedRulesMap.put(r.getId(), r);
		}
		entriesMap = new TreeMap<String,Entry>();
		for (Entry r : getEntries()) {
			entriesMap.put(r.getName(), r);
		}
		
		System.out.println("sharedProfiles " + sharedProfilesMap.size());
		System.out.println("sharedRules " + sharedRulesMap.size());
		System.out.println("entries " + entriesMap.size()); //TODO unsure if needed map
		
	}

	public Map<String, Rule> getSharedRulesMap() {
		return sharedRulesMap;
	}


	public void setSharedRulesMap(Map<String, Rule> sharedRulesMap) {
		this.sharedRulesMap = sharedRulesMap;
	}


	public Map<String, Profile> getSharedProfilesMap() {
		return sharedProfilesMap;
	}


	public void setSharedProfilesMap(Map<String, Profile> sharedProfilesMap) {
		this.sharedProfilesMap = sharedProfilesMap;
	}


	public String getId() {
		return id;
	}


	public void setId(String id) {
		this.id = id;
	}


	public String getRevision() {
		return revision;
	}


	public void setRevision(String revision) {
		this.revision = revision;
	}


	public String getGameSystemId() {
		return gameSystemId;
	}


	public void setGameSystemId(String gameSystemId) {
		this.gameSystemId = gameSystemId;
	}


	public String getGameSystemRevision() {
		return gameSystemRevision;
	}


	public void setGameSystemRevision(String gameSystemRevision) {
		this.gameSystemRevision = gameSystemRevision;
	}


	public String getBattleScribeVersion() {
		return battleScribeVersion;
	}


	public void setBattleScribeVersion(String battleScribeVersion) {
		this.battleScribeVersion = battleScribeVersion;
	}


	public String getName() {
		return name;
	}


	public void setName(String name) {
		this.name = name;
	}


	public String getBooks() {
		return books;
	}


	public void setBooks(String books) {
		this.books = books;
	}


	public String getAuthorName() {
		return authorName;
	}


	public void setAuthorName(String authorName) {
		this.authorName = authorName;
	}


	public String getAuthorContact() {
		return authorContact;
	}


	public void setAuthorContact(String authorContact) {
		this.authorContact = authorContact;
	}


	public String getAuthorUrl() {
		return authorUrl;
	}


	public void setAuthorUrl(String authorUrl) {
		this.authorUrl = authorUrl;
	}


	public List<Entry> getEntries() {
		return entries;
	}


	public void setEntries(List<Entry> entries) {
		this.entries = entries;
	}


	public List<Rule> getRules() {
		return rules;
	}


	public void setRules(List<Rule> rules) {
		this.rules = rules;
	}


	public List<Link> getLinks() {
		return links;
	}


	public void setLinks(List<Link> links) {
		this.links = links;
	}


	public List<Entry> getSharedEntries() {
		return sharedEntries;
	}


	public void setSharedEntries(List<Entry> sharedEntries) {
		this.sharedEntries = sharedEntries;
	}


	public List<EntryGroup> getSharedEntryGroups() {
		return sharedEntryGroups;
	}


	public void setSharedEntryGroups(List<EntryGroup> sharedEntryGroups) {
		this.sharedEntryGroups = sharedEntryGroups;
	}


	public List<Rule> getSharedRules() {
		return sharedRules;
	}


	public void setSharedRules(List<Rule> sharedRules) {
		this.sharedRules = sharedRules;
	}


	public List<Profile> getSharedProfiles() {
		return sharedProfiles;
	}


	public void setSharedProfiles(List<Profile> sharedProfiles) {
		this.sharedProfiles = sharedProfiles;
	}
	
	public String toString(String indent){
		StringBuffer buf = new StringBuffer();
		buf.append(indent + name+"\n");
		if(entries!=null){
		for (Entry  e : entries) {
			buf.append(indent + e.toString(indent+" ")+"\n");
		}
		buf.append(indent + "e " + entries.size()+"\n");
		}
		if(rules!=null){
		for (Rule r : rules) {
			buf.append(indent + r.toString(indent + " ")+"\n");
		}
		buf.append(indent + "r " + rules.size()+"\n");
		}
		if(links!=null){
		for (Link l : links) {
			buf.append(indent + l.toString(indent + " ")+"\n");
		}
		buf.append(indent + "l " + links.size()+"\n");
		}
		if(sharedEntries!=null){
		for (Entry e2 : sharedEntries) {
			buf.append(indent + e2.toString(indent + " ")+"\n");
		}
		buf.append(indent + "e2 " + sharedEntries.size()+"\n");
		}
		if(sharedEntryGroups!=null){
		for (EntryGroup eg : sharedEntryGroups) {
			buf.append(indent + eg.toString(indent + " ")+"\n");
		}
		buf.append(indent + "eg " + sharedEntryGroups.size()+"\n");
		}
		if(sharedRules!=null){
		for (Rule r2 : sharedRules) {
			buf.append(indent + r2.toString(indent+ " ")+"\n");
		}
		buf.append(indent + "r2 " + sharedRules.size()+"\n");
		}
		if(sharedProfiles!=null){
		for (Profile  p : sharedProfiles) {
			buf.append(indent + p.toString(indent + " ")+"\n");
		}
		buf.append(indent + "p " + sharedProfiles.size()+"\n");
		}
		return buf.toString();
	}
}
