package parserXML;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.xml.stream.XMLEventReader;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.events.Attribute;
import javax.xml.stream.events.EndElement;
import javax.xml.stream.events.StartElement;
import javax.xml.stream.events.XMLEvent;

/**
 * xml parser for the entry. creating a entry object identical to the structure in the xml. some fields may be redundant
 * @author harum
 *
 */

public class Entry {
	private String id;
	private String name;
	private float points;
	private String categoryId;
	private String type;
	private int minSelections;
	private int maxSelections;
	private int minInForce;
	private int maxInForce;
	private int minInRoster;
	private int maxInRoster;
	private float minPoints;
	private float maxPoints;
	private boolean collective;
	private boolean hidden;
	private int page;
	
	private List<Entry> entries;
	private List<EntryGroup> entryGroups;
	private List<Modifier> modifiers;
	private List<Rule> rules;
	private List<Link> links;
	private List<Profile> profiles;
	
	
	
	
	List<String> attributes;
	List<String> attributeValues;
	
	List<String> fields;
	
	
	
	
	public Entry(XMLEventReader events, XMLEvent currentEvent) throws XMLStreamException{
		attributes = new ArrayList<String>();
		attributeValues = new ArrayList<String>();
		fields = new ArrayList<String>();
		
		if(currentEvent.isStartElement()){
			StartElement elm = currentEvent.asStartElement();
//			System.out.println("  start -- entry");
			Iterator<Attribute> elementAttributes = elm.getAttributes();
			while(elementAttributes.hasNext()){
				Attribute elementAttribute = elementAttributes.next();
				
				if(elementAttribute.getName().getLocalPart().equals("id")){
					id = elementAttribute.getValue();
				}
				else if(elementAttribute.getName().getLocalPart().equals("name")){
					name = elementAttribute.getValue();
				}
				else if(elementAttribute.getName().getLocalPart().equals("points")){
					points = Float.parseFloat(elementAttribute.getValue());
				}
				else if(elementAttribute.getName().getLocalPart().equals("categoryId")){
					categoryId = elementAttribute.getValue();
				}
				else if(elementAttribute.getName().getLocalPart().equals("type")){
					type = elementAttribute.getValue();
				}
				else if(elementAttribute.getName().getLocalPart().equals("minSelections")){
					minSelections = Integer.parseInt(elementAttribute.getValue());
				}
				else if(elementAttribute.getName().getLocalPart().equals("maxSelections")){
					maxSelections = Integer.parseInt(elementAttribute.getValue());
				}
				else if(elementAttribute.getName().getLocalPart().equals("minInForce")){
					minInForce = Integer.parseInt(elementAttribute.getValue());
				}
				else if(elementAttribute.getName().getLocalPart().equals("maxInForce")){
					maxInForce = Integer.parseInt(elementAttribute.getValue());
				}
				else if(elementAttribute.getName().getLocalPart().equals("minInRoster")){
					minInRoster = Integer.parseInt(elementAttribute.getValue());
				}
				else if(elementAttribute.getName().getLocalPart().equals("maxInRoster")){
					maxInRoster = Integer.parseInt(elementAttribute.getValue());
				}
				else if(elementAttribute.getName().getLocalPart().equals("minPoints")){
					minPoints = Float.parseFloat(elementAttribute.getValue());
				}
				else if(elementAttribute.getName().getLocalPart().equals("maxPoints")){
					maxPoints = Float.parseFloat(elementAttribute.getValue());
				}
				else if(elementAttribute.getName().getLocalPart().equals("collective")){
					collective = Boolean.parseBoolean(elementAttribute.getValue());
				}
				else if(elementAttribute.getName().getLocalPart().equals("hidden")){
					hidden = Boolean.parseBoolean(elementAttribute.getValue());
				}
				else if(elementAttribute.getName().getLocalPart().equals("page")){
					page = Integer.parseInt(elementAttribute.getValue());
				}
				
				
				attributes.add(elementAttribute.getName().getLocalPart());
				attributeValues.add(elementAttribute.getValue());
			}
		}
		
//		System.out.println(attributes);
//		System.out.println(attributeValues);
//		System.out.println(hidden + " " + maxInForce + " " +  minInRoster + " " + minPoints + " " + minSelections + " " +  type + " " +  points + " " +  maxPoints + " " +  minInForce + " " + name + " " +  id + " " +  collective + " " +  categoryId + " " + maxInRoster + " " +  maxSelections + " " + page);
		
		Entries tempEntries = null;
		EntryGroups tempEntryGroups =null;
		Modifiers tempModifiers = null;
		Rules tempRules = null;
		Profiles tempProfiles = null;
		Links tempLinks = null;
		
		while(events.hasNext()){
			XMLEvent event = events.nextEvent();
			
			if(event.isStartElement()){
				StartElement startElm = event.asStartElement();
				fields.add(startElm.getName().getLocalPart());
				if(startElm.getName().getLocalPart().equals("entries")){
					tempEntries = new Entries(events, event);
					entries = tempEntries.getEntries();
				}
				else if(startElm.getName().getLocalPart().equals("entryGroups")){
					tempEntryGroups = new EntryGroups(events, event);
					entryGroups = tempEntryGroups.getEntryGroups();
				}
				else if(startElm.getName().getLocalPart().equals("modifiers")){
					tempModifiers = new Modifiers(events,event);
					modifiers = tempModifiers.getModifiers();
				}
				else if(startElm.getName().getLocalPart().equals("rules")){
					tempRules = new Rules(events, event);
					rules = tempRules.getRules();
				}
				else if(startElm.getName().getLocalPart().equals("profiles")){
					tempProfiles = new Profiles(events,event);
					profiles = tempProfiles.getProfiles();
				}
				else if(startElm.getName().getLocalPart().equals("links")){
					tempLinks = new Links(events,event);
					links = tempLinks.getLinks();
				}
			}
			
			if(event.isEndElement()){
				EndElement endElm = event.asEndElement();
				if(endElm.getName().getLocalPart().equals("entry")){
//					System.out.print(fields);
//					System.out.print("  end -- entry \n");
					break;
				}
			}
		}
		
	}




	public String getId() {
		return id;
	}




	public void setId(String id) {
		this.id = id;
	}




	public String getName() {
		return name;
	}




	public void setName(String name) {
		this.name = name;
	}




	public float getPoints() {
		return points;
	}




	public void setPoints(float points) {
		this.points = points;
	}




	public String getCategoryId() {
		return categoryId;
	}




	public void setCategoryId(String categoryId) {
		this.categoryId = categoryId;
	}




	public String getType() {
		return type;
	}




	public void setType(String type) {
		this.type = type;
	}




	public int getMinSelections() {
		return minSelections;
	}




	public void setMinSelections(int minSelections) {
		this.minSelections = minSelections;
	}




	public int getMaxSelections() {
		return maxSelections;
	}




	public void setMaxSelections(int maxSelections) {
		this.maxSelections = maxSelections;
	}




	public int getMinInForce() {
		return minInForce;
	}




	public void setMinInForce(int minInForce) {
		this.minInForce = minInForce;
	}




	public int getMaxInForce() {
		return maxInForce;
	}




	public void setMaxInForce(int maxInForce) {
		this.maxInForce = maxInForce;
	}




	public int getMinInRoster() {
		return minInRoster;
	}




	public void setMinInRoster(int minInRoster) {
		this.minInRoster = minInRoster;
	}




	public int getMaxInRoster() {
		return maxInRoster;
	}




	public void setMaxInRoster(int maxInRoster) {
		this.maxInRoster = maxInRoster;
	}




	public float getMinPoints() {
		return minPoints;
	}




	public void setMinPoints(float minPoints) {
		this.minPoints = minPoints;
	}




	public float getMaxPoints() {
		return maxPoints;
	}




	public void setMaxPoints(float maxPoints) {
		this.maxPoints = maxPoints;
	}




	public boolean isCollective() {
		return collective;
	}




	public void setCollective(boolean collective) {
		this.collective = collective;
	}




	public boolean isHidden() {
		return hidden;
	}




	public void setHidden(boolean hidden) {
		this.hidden = hidden;
	}




	public int getPage() {
		return page;
	}




	public void setPage(int page) {
		this.page = page;
	}




	public List<Entry> getEntries() {
		return entries;
	}




	public void setEntries(List<Entry> entries) {
		this.entries = entries;
	}




	public List<EntryGroup> getEntryGroups() {
		return entryGroups;
	}




	public void setEntryGroups(List<EntryGroup> entryGroups) {
		this.entryGroups = entryGroups;
	}




	public List<Modifier> getModifiers() {
		return modifiers;
	}




	public void setModifiers(List<Modifier> modifiers) {
		this.modifiers = modifiers;
	}




	public List<Link> getLinks() {
		return links;
	}




	public void setLinks(List<Link> links) {
		this.links = links;
	}




	public List<Rule> getRules() {
		return rules;
	}




	public void setRules(List<Rule> rules) {
		this.rules = rules;
	}




	public List<Profile> getProfiles() {
		return profiles;
	}




	public void setProfiles(List<Profile> profiles) {
		this.profiles = profiles;
	}




	public String toString(String indent) {
		// TODO Auto-generated method stub
		StringBuffer buf = new StringBuffer();
		String indenter = indent + "e ";
		buf.append(indenter + name+"\n");
		if(entries!=null){
			for (Entry  e : entries) {
				buf.append(indenter + e.toString(indent+" ")+"\n");
			}
			buf.append(indenter + "e " + entries.size()+"\n");
		}
		if(entryGroups!=null){
			for (EntryGroup  eg : entryGroups) {
				buf.append(indenter + eg.toString(indent+" ")+"\n");
			}
			buf.append(indenter + "eg " + entryGroups.size()+"\n");
		}
		if(modifiers!=null){
			for (Modifier m : modifiers) {
				buf.append(indenter + m.toString(indent + " ")+"\n");
			}
			buf.append(indenter + "m " + modifiers.size()+"\n");
			}
		if(rules!=null){
			for (Rule r : rules) {
				buf.append(indenter + r.toString(indent + " ")+"\n");
			}
			buf.append(indenter + "r " + rules.size()+"\n");
			}
		if(links!=null){
			for (Link l : links) {
				buf.append(indenter + l.toString(indent + " ")+"\n");
			}
			buf.append(indenter + "l " + links.size()+"\n");
			}
		if(profiles!=null){
			for (Profile p : profiles) {
				buf.append(indenter + p.toString(indent + " ")+"\n");
			}
			buf.append(indenter + "p " + profiles.size());
			}
		return buf.toString();
	}
}
