package parserXML;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.xml.stream.XMLEventReader;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.events.Attribute;
import javax.xml.stream.events.EndElement;
import javax.xml.stream.events.StartElement;
import javax.xml.stream.events.XMLEvent;

public class EntryGroup {
	
	private String id;
	private String name;
	private String defaultEntryId;
	private int minSelections;
	private int maxSelections;
	private int minInForce;
	private int maxInForce;
	private int minInRoster;
	private int maxInRoster;
	private float minPoints;
	private float maxPoints;
	private boolean collective;
	private boolean hidden;
	
	private List<Entry> entries;
	private List<EntryGroup> entryGroups;
	private List<Modifier> modifiers;
	private List<Link> links;
	
	List<String> attributes;
	List<String> attributeValues;
	
	List<String> fields;
	
	public EntryGroup(XMLEventReader events, XMLEvent currentEvent) throws XMLStreamException {
		attributes = new ArrayList<String>();
		attributeValues = new ArrayList<String>();
		fields = new ArrayList<String>();
		
		
		if(currentEvent.isStartElement()){
			StartElement elm = currentEvent.asStartElement();
//			System.out.println("  start -- entryGroup");
			Iterator<Attribute> elementAttributes = elm.getAttributes();
			while(elementAttributes.hasNext()){
				Attribute elementAttribute = elementAttributes.next();
				
				if(elementAttribute.getName().getLocalPart().equals("id")){
					id = elementAttribute.getValue();
				}
				else if(elementAttribute.getName().getLocalPart().equals("name")){
					name = elementAttribute.getValue();
				}
				else if(elementAttribute.getName().getLocalPart().equals("defaultEntryId")){
					defaultEntryId = elementAttribute.getValue();
				}
				else if(elementAttribute.getName().getLocalPart().equals("minSelections")){
					minSelections = Integer.parseInt(elementAttribute.getValue());
				}
				else if(elementAttribute.getName().getLocalPart().equals("maxSelections")){
					maxSelections = Integer.parseInt(elementAttribute.getValue());
				}
				else if(elementAttribute.getName().getLocalPart().equals("minInForce")){
					minInForce = Integer.parseInt(elementAttribute.getValue());
				}
				else if(elementAttribute.getName().getLocalPart().equals("maxInForce")){
					maxInForce = Integer.parseInt(elementAttribute.getValue());
				}
				else if(elementAttribute.getName().getLocalPart().equals("minInRoster")){
					minInRoster = Integer.parseInt(elementAttribute.getValue());
				}
				else if(elementAttribute.getName().getLocalPart().equals("maxInRoster")){
					maxInRoster = Integer.parseInt(elementAttribute.getValue());
				}
				else if(elementAttribute.getName().getLocalPart().equals("minPoints")){
					minPoints = Float.parseFloat(elementAttribute.getValue());
				}
				else if(elementAttribute.getName().getLocalPart().equals("maxPoints")){
					maxPoints = Float.parseFloat(elementAttribute.getValue());
				}
				else if(elementAttribute.getName().getLocalPart().equals("collective")){
					collective = Boolean.parseBoolean(elementAttribute.getValue());
				}
				else if(elementAttribute.getName().getLocalPart().equals("hidden")){
					hidden = Boolean.parseBoolean(elementAttribute.getValue());
				}
				
				
				
				attributes.add(elementAttribute.getName().getLocalPart());
				attributeValues.add(elementAttribute.getValue());
			}
		}
		
//		System.out.println(attributes);
//		System.out.println(attributeValues);
//		System.out.println(hidden + " " + maxInForce + " " +  minInRoster + " " + minPoints + " " + minSelections  + " " +  maxPoints + " " +  minInForce + " " + name + " " +  id + " " +  defaultEntryId + " " +  collective  + " " + maxInRoster + " " +  maxSelections);
		
		Entries tempEntries = null;
		EntryGroups tempEntryGroups =null;
		Modifiers tempModifiers = null;
		Links tempLinks = null;
		
		while(events.hasNext()){
			XMLEvent event = events.nextEvent();
			
			if(event.isStartElement()){
				StartElement startElm = event.asStartElement();
				fields.add(startElm.getName().getLocalPart());
				if(startElm.getName().getLocalPart().equals("entries")){
					tempEntries = new Entries(events, event);
					entries = tempEntries.getEntries();
				}
				else if(startElm.getName().getLocalPart().equals("entryGroups")){
					tempEntryGroups = new EntryGroups(events, event);
					entryGroups = tempEntryGroups.getEntryGroups();
				}
				else if(startElm.getName().getLocalPart().equals("modifiers")){
					tempModifiers = new Modifiers(events,event);
					modifiers = tempModifiers.getModifiers();
				}
				else if(startElm.getName().getLocalPart().equals("links")){
					tempLinks = new Links(events,event);
					links = tempLinks.getLinks();
				}
			}
			
			if(event.isEndElement()){
				EndElement endElm = event.asEndElement();
				if(endElm.getName().getLocalPart().equals("entryGroup")){
//					System.out.print(fields);
//					System.out.print("  end -- entryGroup \n");
					break;
				}
			}
		}
		
	}

	public String getId() {
		return id;
	}

	public void setId(String id) {
		this.id = id;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getDefaultEntryId() {
		return defaultEntryId;
	}

	public void setDefaultEntryId(String defaultEntryId) {
		this.defaultEntryId = defaultEntryId;
	}

	public int getMinSelections() {
		return minSelections;
	}

	public void setMinSelections(int minSelections) {
		this.minSelections = minSelections;
	}

	public int getMaxSelections() {
		return maxSelections;
	}

	public void setMaxSelections(int maxSelections) {
		this.maxSelections = maxSelections;
	}

	public int getMinInForce() {
		return minInForce;
	}

	public void setMinInForce(int minInForce) {
		this.minInForce = minInForce;
	}

	public int getMaxInForce() {
		return maxInForce;
	}

	public void setMaxInForce(int maxInForce) {
		this.maxInForce = maxInForce;
	}

	public int getMinInRoster() {
		return minInRoster;
	}

	public void setMinInRoster(int minInRoster) {
		this.minInRoster = minInRoster;
	}

	public int getMaxInRoster() {
		return maxInRoster;
	}

	public void setMaxInRoster(int maxInRoster) {
		this.maxInRoster = maxInRoster;
	}

	public float getMinPoints() {
		return minPoints;
	}

	public void setMinPoints(float minPoints) {
		this.minPoints = minPoints;
	}

	public float getMaxPoints() {
		return maxPoints;
	}

	public void setMaxPoints(float maxPoints) {
		this.maxPoints = maxPoints;
	}

	public boolean isCollective() {
		return collective;
	}

	public void setCollective(boolean collective) {
		this.collective = collective;
	}

	public boolean isHidden() {
		return hidden;
	}

	public void setHidden(boolean hidden) {
		this.hidden = hidden;
	}

	public List<Modifier> getModifiers() {
		return modifiers;
	}

	public void setModifiers(List<Modifier> modifiers) {
		this.modifiers = modifiers;
	}

	public List<Entry> getEntries() {
		return entries;
	}

	public void setEntries(List<Entry> entries) {
		this.entries = entries;
	}

	public List<EntryGroup> getEntryGroups() {
		return entryGroups;
	}

	public void setEntryGroups(List<EntryGroup> entryGroups) {
		this.entryGroups = entryGroups;
	}

	public List<Link> getLinks() {
		return links;
	}

	public void setLinks(List<Link> links) {
		this.links = links;
	}

	public String toString(String indent) {
		// TODO Auto-generated method stub
		StringBuffer buf = new StringBuffer();
		buf.append(indent + "eg " + name);
		return buf.toString();
	}

}
