package parserXML;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.xml.stream.XMLEventReader;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.events.Attribute;
import javax.xml.stream.events.EndElement;
import javax.xml.stream.events.StartElement;
import javax.xml.stream.events.XMLEvent;

public class Rule {
	
	private String id;
	private String name;
	private boolean hidden;
	private String book;
	private int page;
	
	private String description;
	
	private List<Modifier> modifiers;
	
	
	
	
	List<String> attributes;
	List<String> attributeValues;
	
	List<String> fields;
	
	
	public Rule(XMLEventReader events, XMLEvent currentEvent) throws XMLStreamException {
		attributes = new ArrayList<String>();
		attributeValues = new ArrayList<String>();
		fields = new ArrayList<String>();
		
		if(currentEvent.isStartElement()){
			StartElement elm = currentEvent.asStartElement();
//			System.out.println("  start -- rule");
			Iterator<Attribute> elementAttributes = elm.getAttributes();
			while(elementAttributes.hasNext()){
				Attribute elementAttribute = elementAttributes.next();
				
				if(elementAttribute.getName().getLocalPart().equals("id")){
					id = elementAttribute.getValue();
				}
				else if(elementAttribute.getName().getLocalPart().equals("name")){
					name = elementAttribute.getValue();
				}

				else if(elementAttribute.getName().getLocalPart().equals("hidden")){
					hidden = Boolean.parseBoolean(elementAttribute.getValue());
				}
				else if(elementAttribute.getName().getLocalPart().equals("book")){
					book = elementAttribute.getValue();
				}
				else if(elementAttribute.getName().getLocalPart().equals("page")){
					page = Integer.parseInt(elementAttribute.getValue());
				}
				
				
				attributes.add(elementAttribute.getName().getLocalPart());
				attributeValues.add(elementAttribute.getValue());
			}
		}
		
//		System.out.println(attributes);
//		System.out.println(attributeValues);
//		System.out.println(hidden + " " + book + " " +  name + " " +  id +  " " + page);
		
		
		Modifiers tempModifiers = null;
		Description tempDescription = null;

		while(events.hasNext()){
			XMLEvent event = events.nextEvent();
			
			if(event.isStartElement()){
				StartElement startElm = event.asStartElement();
				fields.add(startElm.getName().getLocalPart());
				if(startElm.getName().getLocalPart().equals("description")){
					tempDescription = new Description(events,event);
					description = tempDescription.getDescription();
				}
				
				else if(startElm.getName().getLocalPart().equals("modifiers")){
					tempModifiers = new Modifiers(events,event);
					modifiers = tempModifiers.getModifiers();
				}
				
			}
			
			if(event.isEndElement()){
				EndElement endElm = event.asEndElement();
				if(endElm.getName().getLocalPart().equals("rule")){
//					System.out.print(fields);
//					System.out.print("  end -- rule \n");
					break;
				}
			}
		}
	}


	public String getId() {
		return id;
	}


	public void setId(String id) {
		this.id = id;
	}


	public String getName() {
		return name;
	}


	public void setName(String name) {
		this.name = name;
	}


	public boolean isHidden() {
		return hidden;
	}


	public void setHidden(boolean hidden) {
		this.hidden = hidden;
	}


	public String getBook() {
		return book;
	}


	public void setBook(String book) {
		this.book = book;
	}


	public int getPage() {
		return page;
	}


	public void setPage(int page) {
		this.page = page;
	}


	public String getDescription() {
		return description;
	}


	public void setDescription(String description) {
		this.description = description;
	}


	public List<Modifier> getModifiers() {
		return modifiers;
	}


	public void setModifiers(List<Modifier> modifiers) {
		this.modifiers = modifiers;
	}
	
	public String toString(String indent){
		StringBuffer buf = new StringBuffer();
		buf.append(indent + "r " +name);
		return buf.toString();
	}
}
